#=========================================================
#   Given an existing MWS authorisation, use this
#       information to retrieve corresponding SP API
#       tokens
#
#   This script should only need to be run once, during the initial
#   cutover from MWS to SP-API
#
#   https://developer-docs.amazon.com/sp-api/docs/authorization-api-v1-reference
#
#=========================================================
use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use JSON;
use URI;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );

use DE_EPAGES::Amazon::API::Constants qw ( $OAUTH_TOKEN_ENDPOINT REPORT_SCHEDULER_PERIOD );

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shop=s'      => \$ShopAlias,
    );

    usage() if $Help;
    usage() unless $StoreName;

    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            my $AmazonConfigClass = LoadClassByAlias('AmazonConfig');
            my @AmazonConfigs = @{ $AmazonConfigClass->objects };

            foreach my $AmazonConfig (@AmazonConfigs) {
                my $PortalSite   = $AmazonConfig->get('PortalSite');
                my $Shop         = $AmazonConfig->getSite;

                if ( $ShopAlias ) {
                    next unless $Shop->alias eq $ShopAlias;
                }

                #-- skip if shop is ClosedByProvider
                next if ($Shop->get('ClosedByProvider'));

                my $SPAPI_AccessToken = $AmazonConfig->get('SPAPI_AccessToken');
                my $SPAPI_AccessTokenExpiry = $AmazonConfig->get('SPAPI_AccessTokenExpiry');
                my $SPAPI_RefreshToken = $AmazonConfig->get('SPAPI_RefreshToken');
                my $SPAPI_RefreshTokenExpiry = $AmazonConfig->get('SPAPI_RefreshTokenExpiry');

                #-- If these values are all set, don't do anything more since the tokens are already there!
                if ( $SPAPI_AccessToken && $SPAPI_AccessTokenExpiry && $SPAPI_RefreshToken && $SPAPI_RefreshTokenExpiry ) {
                    next;
                }

                my $MWSMerchantID = $AmazonConfig->get('MWSMerchantID');
                my $MWSAuthToken = $AmazonConfig->get('MWSAuthToken');

                #-- skip if no merchant id is set or shop is still on SOAP
                next unless defined $MWSMerchantID;

                #-- skip if no MWSAuthToken is set
                next unless defined $MWSAuthToken;

                my $SPAPI_EndPoint = $PortalSite->get('SPAPI_EndPoint');
                next unless defined $SPAPI_EndPoint;

                my $MWSDeveloperAccount = $PortalSite->get('MWSDeveloperAccount');
                #-- Remove dashes?
                $MWSDeveloperAccount =~ s/-//g;
                next unless defined $MWSDeveloperAccount;                

                print "\n[" . $Shop->alias . "]\n";

                my $hPortalInfo = $PortalSite->get(['SPAPI_ClientID', 'SPAPI_ClientSecret']);
                my $ClientID = $hPortalInfo->{'SPAPI_ClientID'};
                my $ClientSecret = $hPortalInfo->{'SPAPI_ClientSecret'};

                #-- The American account does not work anymore ...
                if ( !$ClientID || !$ClientSecret ) {
                    print "  Portal site missing client id/secret, cannot continue\n";
                    next;
                }

                #-- Get a grantless token to use in the call below
                my $Response = $AmazonConfig->doRequest(
                    'POST',
                    "https://api.amazon.com/auth/o2/token?grant_type=client_credentials&client_id=$ClientID&client_secret=$ClientSecret&scope=sellingpartnerapi::migration",
                    '',
                    {'Content-Type' => 'application/json'},
                    {},
                );

                my $AccessToken;
                if ( $Response->is_success ) {
                    my $hInfo = from_json( $Response->decoded_content );
                    $AccessToken = $hInfo->{access_token};
                } else {
                    print '  Grantless token request failed: ' . $Response->decoded_content . "\n";
                    next;
                }

                my $AuthURL = $SPAPI_EndPoint . '/authorization/v1/authorizationCode?sellingPartnerId=' . $MWSMerchantID . '&developerId=' . $MWSDeveloperAccount . '&mwsAuthToken=' . $MWSAuthToken;

                #-- Test getAuthorizationCode
                #-- This returns an authcode that we can swap for an access/refresh token
                my $Response2 = $AmazonConfig->doRequest(
                    'GET',
                    $AuthURL,
                    '',
                    {'Content-Type' => 'application/json', 'x-amz-access-token' => $AccessToken},
                    {
                        'SignRequest' => 1,
                    },
                );

                #-- Swap the authcode for the actual tokens
                if ( $Response2->is_success ) {
                    my $hInfo = from_json( $Response2->decoded_content );
                    my $AuthorizationCode = $hInfo->{payload}->{authorizationCode};

                    #-- Check response and save details into the config objects
                    my $Response3 = $AmazonConfig->doRequest(
                        'POST', 
                        $OAUTH_TOKEN_ENDPOINT, 
                        "grant_type=authorization_code&code=$AuthorizationCode&client_id=$ClientID&client_secret=$ClientSecret", 
                        {'Content-Type' => 'application/x-www-form-urlencoded;charset=UTF-8'},
                        {}
                    );

                    if ( $Response3 && $Response3->is_success() ) {
                        my $DecodedContent = $Response3->decoded_content;

                        my $hJsonResponse;
                        eval { $hJsonResponse = decode_json($DecodedContent); };
                        $hJsonResponse = undef if $@;
                        if ( defined $hJsonResponse ) {
                            $AmazonConfig->set({
                                'SPAPI_AccessToken' => $hJsonResponse->{'access_token'},
                                'SPAPI_AccessTokenExpiry' => time() + $hJsonResponse->{'expires_in'},
                                'SPAPI_RefreshToken' => $hJsonResponse->{'refresh_token'},
                                'SPAPI_RefreshTokenExpiry' => time() + 31104000, #-- 360 days. We shave 4 days off because well, why not give ourselves a bit more leeway?
                            });

                            # activate scheduler
                            $AmazonConfig->activateOrderScheduler(REPORT_SCHEDULER_PERIOD);

                            print "  Success!\n";
                        } else {
                            print '  Failed! Empty response: ' . $DecodedContent . "\n";
                            next;
                        }
                    } else {
                        print '  Failed to retrieve SPAPI token: ' . $Response3->decoded_content . "\n";
                        next;
                    }
                } else {
                    print '  Failed to retrieve authcode: ' . $Response2->decoded_content . "\n";
                    next;
                }

                #-- TODO: Initialise the order report schedule
            }
        },
    );
    my $time2   = time;
    my $seconds = $time2 - $time1;
    my $logmsg  = sprintf( 'need %.3f seconds', $seconds );
    print "\n" . $logmsg . "\n";

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Store)
    -shop         name of shop to process             ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
