use strict;
use Getopt::Long;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error                 qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script                qw ( RunScript );
use DE_EPAGES::Core::API::Warning               qw ( Warning );
use DE_EPAGES::Database::API::Connection        qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Amazon::API::Constants qw ( %AMAZON_PENDING_FEEDS );
use constant NUMBER_OF_SHOWN_DOCUMENTS => 10;
use Data::Dumper;

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $AmazonPlatformAlias;
    my $Feed;
    my $Help;
    my $Document;
    my $aASIN;
    my $InputCreatedSince;
    my $InputCreatedUntil;

    GetOptions(
        'help'               => \$Help,
        'shop=s'             => \$ShopAlias,
        'platform=s'         => \$AmazonPlatformAlias,
        'passwd=s'           => \$Password,
        'storename=s'        => \$StoreName,
        'document=s'         => \$Document,
        'statusfeedid=s'     => \$Feed,
        'ASIN=s@'            => \$aASIN,
        'createdSince=s'     => \$InputCreatedSince,
        'createdUntil=s'     => \$InputCreatedUntil,
    );

    usage() if $Help;
    usage('no storename given') unless $StoreName;

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {
            unless (ExistsCartridgeByPackage('DE_EPAGES::Amazon')) {
                print 'Store: '.$StoreName.": Cartridge Amazon is not installed.\n";
                return;
            }

            _DumpFeedResult($ShopAlias, $AmazonPlatformAlias, $Feed, $Document, $aASIN, $InputCreatedSince, $InputCreatedUntil);
        }
    );
}

sub _DumpFeedResult {
    my ($ShopAlias, $AmazonPlatformAlias, $Feed, $Document, $aASIN, $InputCreatedSince, $InputCreatedUntil) = @_;

    usage('no amazon platform given') unless defined($AmazonPlatformAlias);
    my $Amazon = LoadRootObject->folder('Portals')->child('Amazon');

    usage('invalid amazon platform given') unless ($Amazon->existsChild($AmazonPlatformAlias));

    my $ShopFolder = LoadRootObject()->child('Shops');
    usage("Shop $ShopAlias not found.") unless($ShopFolder->existsChild($ShopAlias));

    my $Shop = $ShopFolder->child($ShopAlias);
    foreach my $PortalConfig (@{$Shop->get('PortalConfigs')}){
        next unless ($PortalConfig->instanceOf('AmazonConfig'));
        next unless ($PortalConfig->get('PortalSite')->alias eq $AmazonPlatformAlias);
        _CallAmazon($PortalConfig, $Feed, $Document, $aASIN, $InputCreatedSince, $InputCreatedUntil);
    }
}

sub _CallAmazon {
    my ($PortalConfig, $Feed, $Document, $aASIN, $InputCreatedSince, $InputCreatedUntil ) = @_;
    eval {
        my $Remote = $PortalConfig->remote();
        my $LastReportExportDateString = $PortalConfig->get("LastReportExportDate") ?
            DateTime::Format::ISO8601->new()->parse_datetime( $PortalConfig->get("LastReportExportDate") )->datetime() : undef;

        my $CreatedSince = $InputCreatedSince ? $InputCreatedSince : $LastReportExportDateString;

        $Remote->is_logging(1);
        if (!$Feed && !$Document && !$aASIN) {
            my $hInputParams;

            if (defined $InputCreatedSince) {
                $hInputParams -> {'createdSince'} = $CreatedSince;
            }

            if (defined $InputCreatedUntil) {
                $hInputParams -> {'createdUntil'} = $InputCreatedUntil;
            }

            $hInputParams -> {'reportTypes'} = $AMAZON_PENDING_FEEDS{'orders'};

            my $Ret2 = $Remote->getAllPendingDocumentInfo($hInputParams);
            my @aReportIDs;
            foreach my $ReportID (@{$Ret2->{'reports'}}) {
                push @aReportIDs, $ReportID->{'reportId'};
            }
            print "\n\ncurrent pending order feeds: \n".join ', ', @aReportIDs if (@{$Ret2->{'reports'}});
        }

        if ($aASIN) {
            my $Summary = $Remote->getLowestOfferList($PortalConfig->get('PortalSite')->get('MWSMarketplaceID'),$aASIN);
            print Dumper $Summary->{'Result'};
        }

        if ($Feed) {
            $Remote->getDocumentProcessingStatus($Feed);
            $Remote->getProcessingReportDocument($Feed);
            print "\n See communication log for further details.\n";
        }

        if ($Document) {
            $Remote->getOrderDocument($Document);
            print "\n See communication log for further details.\n";
        }
        print "\n";
    };

    if (ExistsError()) {
        my $Error = GetError();
        Warning($Error);
    }

}

sub usage {
    my ($ErrorMessage) = @_;
    print (STDERR $ErrorMessage . "\n") if ($ErrorMessage);
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd            database user password           ()
    -storename         name of store                    (Store)
    -platform          amazon platform                  (Amazon_de_DE)
    -shop              shop alias                       ()
                       prints all current pending documents
                       if no feed and no document is specified
    -statusfeedid      pending feed identifier to check   ()
    -document          document id to download            ()
    -ASIN              shows lowest price for this ASIN   ()
                       multiple options of ASIN possible
    -createdSince      The earliest report creation date  ()
                       and time for reports to include
    -createdUntil      The latest report creation date    ()
                       and time for reports to include
flags:
    -help              show the command line options

Example:
    perl $0 -storename Store -platform Amazon_de_DE \
        -shop DemoShop -document amzn1.spdoc.1.3.6242f71e-9365-45f9-b01c-4ca2fad9ed9f.TXWAJVWPNLFVZ.28500

    perl $0 -storename Store -platform Amazon_de_DE \
        -shop DemoShop -createdSince 2022-06-14T00:00:00

    perl $0 -storename Store -platform Amazon_de_DE \
        -shop DemoShop -ASIN B009PKNRGA -ASIN B009PLBLQC


END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
