#==============================================================================
# §package      DE_EPAGES::Amazon::UI::Sysstem
# §state        private
#==============================================================================
package DE_EPAGES::Amazon::UI::System;
use base DE_EPAGES::BacklinkProvider::UI::Provider;

use strict;
use URI;
use URI::URL;
use MIME::Base64;
use Digest::SHA qw ( hmac_sha256_base64 );

use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use DE_EPAGES::Amazon::API::Constants qw ( $OAUTH_STATE_HASH_SECRET );
use DE_EPAGES::Core::API::Error qw ( Error );

sub ErrorDefinition {
    return {
        'NO_STATE' => "State parameter is empty: #State",
        'NO_OBJ_ID' => "ObjectID parameter is empty: #ObjectID",
        'NO_VIEW_ACT' => "ViewAction parameter is empty: #ViewAction",
        'NO_HASH' => "Hash parameter is empty: #Hash",
        'INV_HASH' => "Backlink hash does not validate (#Hash != #GeneratedHash)",
   };
}

#========================================================================================
# §function     $self->BacklinkAmazonOnboarding
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       BacklinkAmazonOnboarding($Servlet);
#----------------------------------------------------------------------------------------
# §description  The subroutine for the change action used as the redirect_uri for Amazon's OAuth process
#               See also: DE_EPAGES::Amazon::UI::AmazonConfig::ViewGeneral
#----------------------------------------------------------------------------------------
# §input        $Servlet | Servlet object | object
#========================================================================================
sub BacklinkAmazonOnboarding {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form();
    my $hQueryParameters = $Form->queryValues();

    #-- Original Change action must be removed, which is this subroutine's change action
    delete($hQueryParameters->{'ChangeAction'});

    #-- state contains the URL which is the backlink to the MBO. No need to send it with QueryString again
    my $aState = delete($hQueryParameters->{'state'});

    #-- We MUST have a state value, which is non empty
    Error('NO_STATE', { 'State' => $aState }) unless $aState;

    #-- The state parameter was base64 encoded
    my $URL = URI->new( decode_base64($aState->[0]) );

    #-- Ensure that special characters used in the url path/query are url encoded. E.g. + in the query portion gets replaced by ' ' unless uri encoded
    my $hBacklinkUrlParameters = ParseQueryString($URL->query(), 'utf-8');

    my $ObjectID = $hBacklinkUrlParameters->{'ObjectID'}[0];
    Error('NO_OBJ_ID', { 'ObjectID' => $ObjectID }) unless $ObjectID =~ /\S/;

    my $ViewAction = $hBacklinkUrlParameters->{'ViewAction'}[0];
    Error('NO_VIEW_ACT', { 'ViewAction' => $ViewAction }) unless $ViewAction =~ /\S/;

    my $Hash = $hBacklinkUrlParameters->{'Hash'}[0];
    Error('NO_HASH', { 'Hash' => $Hash }) unless $Hash =~ /\S/;

    #-- Remove the hash parameter
    delete($hBacklinkUrlParameters->{'Hash'});

    my $VerifyHost = $URL->host;

    my $GeneratedHash = hmac_sha256_base64( "$VerifyHost-$ObjectID-$ViewAction", $OAUTH_STATE_HASH_SECRET);
    Error('INV_HASH', { 'Hash' => $Hash, 'GeneratedHash' => $GeneratedHash }) unless $Hash eq $GeneratedHash;

    my $hQueryParam = {%{$hBacklinkUrlParameters}, %{$hQueryParameters}};
    $URL->query_form($hQueryParam);

    return $self->SUPER::RedirectToURL( $URL->as_string() );
}

1;
