#========================================================================================
# package      DE_EPAGES::Amazon::t::RemoteMWSToMerchantTest
# base         Test::Class
# state        private
#----------------------------------------------------------------------------------------
# description  Tests for RemoteMWSToMerchant adapter using mocks
#========================================================================================
package DE_EPAGES::Amazon::t::RemoteMerchantToMWSAdapterTest;
use base Test::Class;

use DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Amazon::API::Constants qw ( %AMAZON_PENDING_FEEDS );
use Test::More;
use Test::MockObject;
use HTTP::Response;
use strict;

# setup methods are run before every test method.
sub setUp : Test(setup) {
    my $self = shift;

    $self->{MWSMock}  = Test::MockObject->new();
    $self->{HTTPMock} = Test::MockObject->new();
    $self->{Remote}   = DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter->new(
        AWSURL         => 1,
        AWSAccessKeyId => 2,
        AWSSecretKeyId => 3,
        MerchantID     => 4,
        MWSAuthToken   => 5,
        MarketplaceID  => 6,
        SPAPI_EndPoint => 7,
        mws            => $self->{MWSMock}
    );
}

sub testCredentials : Test(7) {
    my $self = shift;
    is( $self->{Remote}->url,               1, 'AWSURL' );
    is( $self->{Remote}->marketplace,       6, 'marketplace' );
    is( $self->{Remote}->merchant,          4, 'merchant' );
    is( $self->{Remote}->mwsauthtoken,      5, 'merchant' );
    is( $self->{Remote}->spapi_endpoint,    7, 'spapi endpoint existing' );
    ok( !$self->{Remote}->is_logging, 'no logging' );
    ok( $self->{Remote}->mws,         'mws adapter existing' );
}

sub testLogin : Test(1) {
    my $self = shift;
    $self->{MWSMock}->mock(
        'getOrderServiceStatus',
        sub {
            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0" ?>
<root />
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );
    $self->{Remote}->login();
    $self->{MWSMock}->called_ok('getOrderServiceStatus');
}

sub testGetAllPendingDocumentInfo : Test(2) {
    my $self = shift;
    $self->_prepareMWSMockForGetAllPendingDocumentInfo();

    my $hInputParams -> {'reportTypes'} = $AMAZON_PENDING_FEEDS{'orders'};

    my $Ret = $self->{Remote}->getAllPendingDocumentInfo($hInputParams);
    $self->{MWSMock}->called_ok('getReports');

    my @aReportIDs;
    foreach my $ReportID (@{$Ret->{'reports'}}) {
        push @aReportIDs, $ReportID->{'reportId'};
    }

    my $ReportIDs = join ', ', @aReportIDs;
    is_deeply( $ReportIDs, "72969019157, 72958019157" , 'report ids.' );
}

sub _prepareMWSMockForGetAllPendingDocumentInfo {
    my $self = shift;
    $self->{MWSMock}->mock(
        'getReports',
        sub {
            my $hReturnGetReports = {'reports' => [
                {
                    'processingStatus' => 'DONE',
                    'dataEndTime' => '2022-06-14T13:54:29+00:00',
                    'reportScheduleId' => '50030019157',
                    'createdTime' => '2022-06-14T13:55:07+00:00',
                    'reportDocumentId' => 'amzn1.spdoc.1.3.6242f71e-9365-45f9-b01c-4ca2fad9ed9f.TXWAJVWPNLFVZ.28500',
                    'processingEndTime' => '2022-06-14T13:55:23+00:00',
                    'marketplaceIds' => [
                        'A1PA6795UKMFR9'
                    ],
                    'reportType' => 'GET_ORDER_REPORT_DATA_INVOICING',
                    'reportId' => '72969019157',
                    'processingStartTime' => '2022-06-14T13:55:13+00:00',
                    'dataStartTime' => '2022-06-14T13:39:29+00:00'
                },
                {
                    'processingStatus' => 'DONE',
                    'dataEndTime' => '2022-06-14T12:26:05+00:00',
                    'reportScheduleId' => '50029019157',
                    'createdTime' => '2022-06-14T12:26:13+00:00',
                    'reportDocumentId' => 'amzn1.spdoc.1.3.594f527d-4dd7-49d9-86f7-7c7dc0f78c2b.T20GLRP4GIFI70.28500',
                    'processingEndTime' => '2022-06-14T12:26:29+00:00',
                    'marketplaceIds' => [
                        'A1PA6795UKMFR9'
                    ],
                    'reportType' => 'GET_ORDER_REPORT_DATA_INVOICING',
                    'reportId' => '72958019157',
                    'processingStartTime' => '2022-06-14T12:26:18+00:00',
                    'dataStartTime' => '2022-06-14T12:11:05+00:00'
                }
            ]};

            return $hReturnGetReports;
        }
    );

}

print "!!!!\n testGetDocumentProcessingStatus,\n testGetProcessingReportDocument, \n testPostDocument needs refactoring when new Feeds API for SPAPI is implemented\n!!!!\n";
# Needs to be adjusted when the SubmitFeed is changed to use SPAPI.
sub testGetDocumentProcessingStatus : Test(2) {
    my $self  = shift;
    my $DocId = 'DOCUMENT_1';
    $self->{MWSMock}->mock(
        'getFeedSubmissionList',
        sub {
            shift;
            my (%Info) = @_;
            is( $Info{'FeedSubmissionIdList.Id.1'}, $DocId, 'document id' );

            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0" ?>
<root />
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );
    $self->{Remote}->getDocumentProcessingStatus($DocId);
    $self->{MWSMock}->called_ok('getFeedSubmissionList');
}

# Needs to be adjusted when the SubmitFeed is changed to use SPAPI.
sub testGetProcessingReportDocument : Test(2) {
    my $self  = shift;
    my $DocId = 'DOCUMENT_1';
    $self->{MWSMock}->mock(
        'getFeedSubmissionResult',
        sub {
            shift;
            my (%Info) = @_;
            is( $Info{'FeedSubmissionId'}, $DocId, 'document id' );

            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0" ?>
<root />
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );
    $self->{Remote}->getProcessingReportDocument($DocId);
    $self->{MWSMock}->called_ok('getFeedSubmissionResult');
}

sub testGetOrderDocument : Test(1) {
    my $self  = shift;
    my $DocId = 'amzn1.spdoc.1.3.6242f71e-9365-45f9-b01c-4ca2fad9ed9f.TXWAJVWPNLFVZ.28500';
    my $ReportId = '72969019157';

    $self->_prepareMWSMockForGetOrderDocument();

    $self->{Remote}->getOrderDocument($DocId, $ReportId);
    $self->{MWSMock}->called_ok('getReportDocument');
}

sub _prepareMWSMockForGetOrderDocument {
    my $self = shift;
    $self->{MWSMock}->mock(
        'getReportDocument',
        sub {
                    return << 'EOL';
<?xml version="1.0" encoding="UTF-8"?>
<AmazonEnvelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="amzn-envelope.xsd">
  <Header>
    <DocumentVersion>1.01</DocumentVersion>
      <MerchantIdentifier>A2V5YPL4FRBD8D</MerchantIdentifier>
  </Header>
  <MessageType>OrderReport</MessageType>
<Message>
    <MessageID>1</MessageID>
    <OrderReport>
        <AmazonOrderID>304-3098396-5473110</AmazonOrderID>
        <AmazonSessionID>304-3098396-5473110</AmazonSessionID>
        <OrderDate>2022-06-14T13:24:15+00:00</OrderDate>
        <OrderPostedDate>2022-06-14T13:24:15+00:00</OrderPostedDate>
        <BillingData>
            <BuyerEmailAddress>4p249wzqfn9v84w@marketplace.amazon.de</BuyerEmailAddress>
            <BuyerName>ePages GmbH</BuyerName>
            <BuyerPhoneNumber>040350188123</BuyerPhoneNumber>
            <Address>
                <Name>QA Test Kufer</Name>
                <AddressFieldOne>ePages GmbH</AddressFieldOne>
                <AddressFieldTwo>Heinrich-Heine-Str. 1</AddressFieldTwo>
                <City>Jena</City>
                <StateOrRegion>Thringen</StateOrRegion>
                <PostalCode>07749</PostalCode>
                <CountryCode>DE</CountryCode>
            </Address>
        </BillingData>
        <FulfillmentData>
            <FulfillmentMethod>Ship</FulfillmentMethod>
            <FulfillmentServiceLevel>Standard</FulfillmentServiceLevel>
            <Address>
                <Name>QA Test Kufer</Name>
                <AddressFieldOne>ePages GmbH</AddressFieldOne>
                <AddressFieldTwo>Heinrich-Heine-Str. 1</AddressFieldTwo>
                <City>Jena</City>
                <StateOrRegion>Thringen</StateOrRegion>
                <PostalCode>07749</PostalCode>
                <CountryCode>DE</CountryCode>
                <PhoneNumber>040350188123</PhoneNumber>
            </Address>
        </FulfillmentData>
        <IsBusinessOrder>false</IsBusinessOrder>
        <IsIba>false</IsIba>
        <Item>
            <AmazonOrderItemCode>19856487337611</AmazonOrderItemCode>
            <SKU>1001[1]</SKU>
            <Title>Linsenkopfschrauben mit Flansch und Innensechsrund (ISR) - M3x5 - (20 Stck) - ISO 7380-2 TX - rostfreier Edelstahl A2 V2A - Vollgewinde - Flanschschrauben - Flachkopfschrauben - SC7380-2TX</Title>
            <Quantity>1</Quantity>
            <ProductTaxCode>PTC_PRODUCT_TAXABLE_A</ProductTaxCode>
            <ItemPrice>
               <Component>
                  <Type>Principal</Type>
                  <Amount currency="EUR">15.00</Amount>
               </Component>
               <Component>
                  <Type>Shipping</Type>
                  <Amount currency="EUR">0.00</Amount>
               </Component>
               <Component>
                  <Type>Tax</Type>
                  <Amount currency="EUR">0.00</Amount>
               </Component>
               <Component>
                  <Type>ShippingTax</Type>
                  <Amount currency="EUR">0.00</Amount>
               </Component>
            </ItemPrice>
            <ItemFees>
               <Fee>
                  <Type>Commission</Type>
                  <Amount currency="EUR">-1.95</Amount>
               </Fee>
            </ItemFees>
         </Item>
    </OrderReport>
 </Message>
</AmazonEnvelope>
EOL
        }
    );
}

# Needs to be adjusted when the SubmitFeed is changed to use SPAPI.
sub testPostDocument : Test(3) {
    my $self     = shift;
    my $FeedType = '_TEST_';
    $self->{MWSMock}->mock(
        'submitFeed',
        sub {
            shift;
            my (%Info) = @_;
            is( $Info{'FeedType'},        $FeedType, 'feedType' );
            is( ref( $Info{'rContent'} ), 'REF',     'content' );

            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0"?>
<SubmitFeedResponse xmlns="http://mws.amazonaws.com/doc/2009-01-01/"><SubmitFeedResult><FeedSubmissionInfo><FeedSubmissionId>2533151448</FeedSubmissionId><FeedType>_POST_OFFER_ONLY_DATA_</FeedType><SubmittedDate>2012-01-03T10:39:10+00:00</SubmittedDate><FeedProcessingStatus>_SUBMITTED_</FeedProcessingStatus></FeedSubmissionInfo></SubmitFeedResult><ResponseMetadata><RequestId>7a432b03-df21-4870-9f5e-bbdad8a1a78e</RequestId></ResponseMetadata></SubmitFeedResponse>
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );
    $self->{Remote}->postDocument( $FeedType, \"TestData" );
    $self->{MWSMock}->called_ok('submitFeed');
}

package main;    ## no critic (ProhibitMultiplePackages)
use DE_EPAGES::Core::API::Script qw ( RunScript );

RunScript(
    Sub => sub {
        Test::Class->runtests;
    }
);

1;
