#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelResponse::Parser
# state        public
# description  Description
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelResponse::Parser;

use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::Constants qw ( $STATUS_CODE_OK );

use strict;


#========================================================================================
sub new {
    my $class = shift;

    my $self = bless {}, $class;
    return $self;
}

#========================================================================================
# function     parseResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       parseResponse( $SOAPResponse );
#----------------------------------------------------------------------------------------
# description  From the XML response of the webservice retrives the most important
#               information, and stores it in an easy to access hash.
#----------------------------------------------------------------------------------------
# input        $SOAPResponse   | SOAP response, must contain the XML the parse | Object
# return       $aParsedResponse | array with the relevant values of the response | ref.array
#========================================================================================
sub parseResponse {
    my $self = shift;
    my ($SOAPResponse) = @_;

    if ( defined $SOAPResponse->{'Fault'} &&
         defined $SOAPResponse->{'Fault'}->{'detail'} &&
         defined $SOAPResponse->{'Fault'}->{'detail'}->{'errorDetails'}
    ) {
        my $ErrorCode = $SOAPResponse->{'Fault'}->{'detail'}->{'errorDetails'}->{'code'};
        my $ErrorDescription = $SOAPResponse->{'Fault'}->{'detail'}->{'errorDetails'}->{'description'};
        $self->_Warning(
                'parseResponse: DHLRespondedWithError',
                {
                    'ErrorCode'    => $ErrorCode,
                    'ErrorMessage' => $ErrorDescription,
                    'Reason'       => $ErrorDescription,
                }
        );

        return [{
                'IsError' => 1,
                'StatusCode' => $ErrorCode,
                'StatusText' => 'Fault',
                'StatusMessage' => [$ErrorDescription]
        }];
    }

    my $SOAPResponseInfo = $SOAPResponse->{'part1'};
    return $self->_parseCreateShipmentOrder( $SOAPResponseInfo );
}

#========================================================================================
# function     _parseCreateShipmentOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       _parseCreateShipmentOrder( $SOAPResponseInfo );
#----------------------------------------------------------------------------------------
# description  Parses SOAP response data from shipping order generation web service.
#----------------------------------------------------------------------------------------
# input        $SOAPResponseInfo    | hash with SOAP response | hasref
# return       $aParsedResponse | array with parsed response | arrayref
#========================================================================================
sub _parseCreateShipmentOrder {
    my $self = shift;
    my ($SOAPResponseInfo) = @_;

    my $aParsedResponse = [];

    my @aCreationStates;
    if (isValid($SOAPResponseInfo->{'CreationState'})) {
        @aCreationStates = @{ $SOAPResponseInfo->{'CreationState'}};
    } else {
        my $Status = $SOAPResponseInfo->{'Status'};
        my $StatusCode = $Status->{'statusCode'};
        my $StatusText = $Status->{'statusText'};
        my $aStatusMessage = $Status->{'statusMessage'};
        $self->_checkStatusCodeOK( $StatusCode, $StatusText );
        push( @$aParsedResponse, {
                'IsError' => 1,
                'StatusCode' => $StatusCode,
                'StatusText' => $StatusText,
                'StatusMessage' => $aStatusMessage,
        });
    }

    foreach my $CreationState ( @aCreationStates ) {
        my $StatusCode = $CreationState->{LabelData}->{Status}->{'statusCode'};
        my $StatusText = $CreationState->{LabelData}->{Status}->{'statusText'};
        my $aStatusMessage = $CreationState->{LabelData}->{Status}->{'statusMessage'};
        $self->_checkStatusCodeOK( $StatusCode, $StatusText );

        my $aInvalidAddressMsgs = grep { $_ =~ /Warning\: The address could not be validated correctly/ } @{ $CreationState->{LabelData}->{Status}->{'statusMessage'} };
        my $invalidAdressBool = ( defined($aInvalidAddressMsgs) and scalar($aInvalidAddressMsgs) > 0 );
        my $hVal = {
            'StatusCode'     => $StatusCode,
            'StatusText'  => $StatusText,
            'StatusMessage'  => $aStatusMessage,
            'InvalidAddress' => $invalidAdressBool,
            'SequenceNumber' => $CreationState->{'sequenceNumber'},
            'ShipmentNumber' => $CreationState->{'shipmentNumber'},
            'Labelurl'       => $CreationState->{'LabelData'}->{'labelUrl'},
        };
        push( @$aParsedResponse, $hVal );
    }

    return $aParsedResponse;
}

#========================================================================================
# function     _checkStatusCodeOK
# state        private
#----------------------------------------------------------------------------------------
# syntax       _checkStatusCodeOK( $StatusCode, $StatusText );
#----------------------------------------------------------------------------------------
# description  Throws an error unless status code does not report any error.
#----------------------------------------------------------------------------------------
# input        $StatusCode    | SOAP response status code    | Integer
# input        $StatusText | SOAP response status message | String
#========================================================================================
sub _checkStatusCodeOK {
    my $self = shift;
    my ($StatusCode, $StatusText) = @_;

    return if ( $StatusCode eq $STATUS_CODE_OK );

    $self->_Warning(
            '_checkStatusCodeOK:DHLRespondedWithError',
            {
                'ErrorCode' => $StatusCode,
                'Code'      => $StatusCode,
                'Parameter' => $StatusText,
                'Reason'    => $StatusText,
            }
    );

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _Warning {
    my $self = shift;
    my ($WarningMessage, $WarningData) = @_;

    Warning($WarningMessage, $WarningData);
}

1;
