#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Builder
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Builder module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Builder;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify;
use Test::More;
use Test::Exception;

use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder;

use strict;

use constant TEST_AUTHENTIFICATION_USER => 'MyTestUser';
use constant TEST_AUTHENTIFICATION_SIGNATURE => 'MyTestSignature';
use constant TEST_AUTHENTIFICATION_HASH => {
        'header' => {
            'user'      => TEST_AUTHENTIFICATION_USER,
            'signature' => TEST_AUTHENTIFICATION_SIGNATURE,
        }
};
use constant TEST_VERSION_HASH => {
        'Version' => {
            'majorRelease' => 3,
            'minorRelease' => 3,
        }
};
use constant EMPTY_SHIPMENT_ORDER_ASSEMBLY => {
    'nodeData' => { 'ShipmentOrder' => [{}] },
    'nodeWarnings' => []
};

## no critic (ProtectPrivateSubs)
## no critic (ProhibitPrivateSubroutineUse)

#========================================================================================
sub test {
    my $self = shift;

    $self->assembleAuthentification_test();
    $self->assembleVersion_test();
    $self->createShipmentOrder_positive_test();
    $self->createShipmentOrder_negative_test();
    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub assembleAuthentification_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship();
    my $hAutentification =
    DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder->_assembleAuthentification($mockedShippingLabel);

    is_deeply($hAutentification, TEST_AUTHENTIFICATION_HASH, "$SubTestName - Authentification hash assembled");
}

#========================================================================================
sub assembleVersion_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hVersion = DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder->_assembleVersion();

    is_deeply($hVersion, TEST_VERSION_HASH, "$SubTestName - Version hash assembled");
}

#========================================================================================
sub createShipmentOrder_positive_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship();
    my $mockedBuilder = $self->_createBuilder();
    my $hShipmentOrder = $mockedBuilder->_createShipmentOrder($mockedShippingLabel);

    my $hExpectedAuthentification = TEST_AUTHENTIFICATION_HASH;
    my $hExpectedVersion = TEST_VERSION_HASH;

    is_deeply($hShipmentOrder,
        {
            %{$hExpectedAuthentification},
            'part1' => {
                %{$hExpectedVersion},
                'ShipmentOrder' => [{}]
            }
        },
        "$SubTestName - ShipmentOrder hash assembled");
}

#========================================================================================
sub createShipmentOrder_negative_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    throws_ok { DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder->_createShipmentOrder() }
       qr/PARAMETER_MISSING/,
       "$SubTestName - Authentification hash not assembled. No shipping label has been passed";

    my $mockedNotDHLVersandShippingLabel = $self->_createShippingLabelNoDHLIntraship();
    throws_ok { DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder->_createShipmentOrder($mockedNotDHLVersandShippingLabel) }
        qr/WRONG_CLASS/,
        "$SubTestName - Authentification hash not assembled. No proper DHLIntraship shipping label has been passed";

}


#========================================================================================
# Mocking methods
#========================================================================================
sub _createBuilder {
    my $self = shift;

    my $aParameterList = [];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder',
        $aParameterList
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleShipmentOrder',
            EMPTY_SHIPMENT_ORDER_ASSEMBLY,
            [ { 'object' => 'DE_EPAGES::ExternalShipping::API::Object::ShippingLabel' } ]
    );
    $Mockify->addMock(
            '_LogShippingFulfillment',
            sub { return; }
    );

    return $Mockify->getMockObject();
}

sub _createShippingLabelDHLIntraship {
    my $self = shift;

    my $aParameterList = [
            'ShippingFulfillmentMethod' => $self->_createShippingFulfillmentMethodDHLIntraship(),
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

sub _createShippingFulfillmentMethodDHLIntraship {
    my $self = shift;

    my $aParameterList = [
            'WebserviceUsername' => TEST_AUTHENTIFICATION_USER,
            'WebservicePassword' => TEST_AUTHENTIFICATION_SIGNATURE,
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

sub _createShippingLabelNoDHLIntraship {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::ExternalShipping::API::Object::ShippingLabel',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript;
