#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelResponse_Parser
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Builder module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelResponse_Parser;
use base DE_EPAGES::Core::API::Test;

use strict;

use Test::Mockify::Sut;
use Test::More;

use constant STATUS_CODE_RESPONSE_SUCCESS_AND_WARNING => '0';
use constant STATUS_CODE_RESPONSE_ERROR => '1101';
use constant STATUS_TEXT_RESPONSE_SUCCESS => 'ok';
use constant STATUS_TEXT_RESPONSE_WARNING => 'Weak validation error occured.';
use constant STATUS_TEXT_RESPONSE_ERROR => 'Hard validation error occured.';
use constant STATUS_MESSAGE_RESPONSE_SUCCESS => 'Der Webservice wurde ohne Fehler ausgefhrt.';
use constant STATUS_MESSAGE_RESPONSE_WARNING => 'Der Ort ist zu dieser PLZ nicht bekannt. Die Sendung ist trotzdem leitcodierbar und es wurde ein Versandschein erzeugt.';
use constant STATUS_MESSAGE_RESPONSE_ERROR => 'Die ausgewhlte Abrechnungsnummer steht nicht zur Verfgung.';
use constant LABEL_URL => 'https://cig.dhl.de/gkvlabel/SANDBOXAZURE/dhl-vls/gw/shpmntws/printShipment?token=x5xzrHE7ct';
use constant SHIPMENT_NUMBER => '222201010014152087';
use constant SEQUENCE_NUMBER => '23356';
use constant CREATE_SHIPMENT_ORDER_RESPONSE_SUCCESS => {
    'part1' => {
    'Version' => { 'minorRelease' => '0', 'majorRelease' => '3' },
    'Status' => {
      'statusText' => STATUS_TEXT_RESPONSE_SUCCESS,
      'statusCode' => STATUS_CODE_RESPONSE_SUCCESS_AND_WARNING
    },
    'CreationState' => [
      {
        'shipmentNumber' => SHIPMENT_NUMBER,
        'sequenceNumber' => SEQUENCE_NUMBER,
        'LabelData' => {
          'labelUrl' => LABEL_URL,
          'Status' => {
            'statusMessage' => [ STATUS_MESSAGE_RESPONSE_SUCCESS ],
            'statusText' => STATUS_TEXT_RESPONSE_SUCCESS,
            'statusCode' => STATUS_CODE_RESPONSE_SUCCESS_AND_WARNING
          }
        }
      }
    ]
  }
};
use constant CREATE_SHIPMENT_ORDER_RESPONSE_WARNING => {
    'part1' => {
    'Version' => { 'minorRelease' => '0', 'majorRelease' => '3' },
    'Status' => {
      'statusText' => STATUS_TEXT_RESPONSE_WARNING,
      'statusCode' => STATUS_CODE_RESPONSE_SUCCESS_AND_WARNING
    },
    'CreationState' => [
      {
        'shipmentNumber' => SHIPMENT_NUMBER,
        'sequenceNumber' => SEQUENCE_NUMBER,
        'LabelData' => {
          'labelUrl' => LABEL_URL,
          'Status' => {
            'warningMessage' => [
              {
                'statusElement' => 'address.receiver',
                'statusMessage' => STATUS_MESSAGE_RESPONSE_WARNING
              }
            ],
            'statusMessage' => [ STATUS_MESSAGE_RESPONSE_WARNING ],
            'statusText' => STATUS_TEXT_RESPONSE_WARNING,
            'statusType' =>'Warning',
            'statusCode' => STATUS_CODE_RESPONSE_SUCCESS_AND_WARNING
          }
        }
      }
    ]
  }
};
use constant CREATE_SHIPMENT_ORDER_RESPONSE_ERROR => {
    'part1' => {
    'Version' => { 'minorRelease' => '0', 'majorRelease' => '3' },
    'Status' => {
      'statusText' => STATUS_TEXT_RESPONSE_ERROR,
      'statusCode' => STATUS_CODE_RESPONSE_ERROR
    },
    'CreationState' => [
      {
        'sequenceNumber' => SEQUENCE_NUMBER,
        'LabelData' => {
          'labelUrl' => LABEL_URL,
          'Status' => {
            'errorMessage' => [
                {
                   'statusElement' => 'ShipmentOrder.Shipment.ShipmentDetails.cis:accountNumber',
                   'statusMessage' => STATUS_MESSAGE_RESPONSE_ERROR
                }
            ],
            'statusMessage' => [ STATUS_MESSAGE_RESPONSE_ERROR ],
            'statusText' => STATUS_TEXT_RESPONSE_ERROR,
            'statusType' =>'Error',
            'statusCode' => STATUS_CODE_RESPONSE_ERROR
          }
        }
      }
    ]
  }
};

sub test {
    my $self = shift;

    $self->test_DHLIntrashipSuccessResponseParsing();
    $self->test_DHLIntrashipWarningResponseParsing();
    $self->test_DHLIntrashipErrorResponseParsing();

    done_testing();
}

sub test_DHLIntrashipSuccessResponseParsing {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT = $self->_createSUT();
    my $ParsedResponse = $SUT->parseResponse(CREATE_SHIPMENT_ORDER_RESPONSE_SUCCESS);

    is_deeply(
        $ParsedResponse,
        [{
            'StatusCode' => STATUS_CODE_RESPONSE_SUCCESS_AND_WARNING,
            'StatusText'  => STATUS_TEXT_RESPONSE_SUCCESS,
            'StatusMessage'  => [ STATUS_MESSAGE_RESPONSE_SUCCESS ],
            'InvalidAddress' => '',
            'SequenceNumber' => SEQUENCE_NUMBER,
            'ShipmentNumber' => SHIPMENT_NUMBER,
            'Labelurl'       => LABEL_URL,
        }],
        "$SubTestName - Successful response gets properly parsed");
}

sub test_DHLIntrashipWarningResponseParsing {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT = $self->_createSUT();
    my $ParsedResponse = $SUT->parseResponse(CREATE_SHIPMENT_ORDER_RESPONSE_WARNING);

    is_deeply(
        $ParsedResponse,
        [{
            'StatusCode' => STATUS_CODE_RESPONSE_SUCCESS_AND_WARNING,
            'StatusText'  => STATUS_TEXT_RESPONSE_WARNING,
            'StatusMessage'  => [ STATUS_MESSAGE_RESPONSE_WARNING ],
            'InvalidAddress' => '',
            'SequenceNumber' => SEQUENCE_NUMBER,
            'ShipmentNumber' => SHIPMENT_NUMBER,
            'Labelurl'       => LABEL_URL,
        }],
        "$SubTestName - Warning response gets properly parsed");
}

sub test_DHLIntrashipErrorResponseParsing {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT = $self->_createSUT();
    my $ParsedResponse = $SUT->parseResponse(CREATE_SHIPMENT_ORDER_RESPONSE_ERROR);

    is_deeply(
        $ParsedResponse,
        [{
            'StatusCode' => STATUS_CODE_RESPONSE_ERROR,
            'StatusText'  => STATUS_TEXT_RESPONSE_ERROR,
            'StatusMessage'  => [ STATUS_MESSAGE_RESPONSE_ERROR ],
            'InvalidAddress' => '',
            'SequenceNumber' => SEQUENCE_NUMBER,
            'ShipmentNumber' => undef,
            'Labelurl'       => LABEL_URL,
        }],
        "$SubTestName - Error response gets properly parsed");
}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelResponse::Parser');
    $MockifySut->mockImported('DE_EPAGES::Core::API::Warning', 'Warning')
        ->whenAny()
        ->thenReturnUndef();;

    return $MockifySut->getMockObject();
}

__PACKAGE__->runScript();
