#========================================================================================
# package      DE_EPAGES::MolliePayments::API::TLE::MolliePaymentsTLEHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Implements TLE function
#----------------------------------------------------------------------------------------
# example
#========================================================================================
package DE_EPAGES::MolliePayments::API::TLE::MolliePaymentsTLEHandler;
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::MolliePayments::API::Constants qw ( @DISPLAY_PAYMENT_ICONS @MOLLIE_LIMITED_PAYMENTMETHODS);

use strict;

use strict;


#==================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------
# description  Function registers the TLE handler process.
#----------------------------------------------------------------------------------
# input        $hParams | hook arguments keys: Processor | hash reference
#==================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register($Params->{'Processor'});
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::MolliePayments::API::TLE::MolliePaymentsTLEHandler;
# example      my $Handler  = new DE_EPAGES::MolliePayments::API::TLE::MolliePaymentsTLEHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#==================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------
# description  Function registers this handle on the processor.
#----------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#==================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'MolliePaymentsTLEHandler.RegisterHandlerProc' );

    $Processor->registerHandler( 'FunctionHandler', $self, 'MollieAdded' );
    $Processor->registerHandler( 'FunctionHandler', $self, 'MollieDisplayIconsAlways');
    $Processor->registerHandler( 'FunctionHandler', $self, 'MollieLimitedPaymentMethods');
    $Processor->registerHandler( 'FunctionHandler', $self, 'MollieDisplayLocalizedIcons');

    return;
}

#========================================================================================
# function     MollieAdded
# state        public
#----------------------------------------------------------------------------------------
# syntax       #MollieAdded
#----------------------------------------------------------------------------------------
# description  Checks if a Mollie payment method is already added to shop.
#----------------------------------------------------------------------------------------
# input        #Processor | tle processor | object
# input        $aParams | #Shop.Object | array
# return       #Result | true if PaymentMethod was already added else false | boolean
#========================================================================================
sub MollieAdded {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Shop) = @$aParams;

    my $PaymentMethods  = $Shop->get('PaymentMethods');
    my $MollieExists = 0;

    foreach my $PaymentMethodItem (@$PaymentMethods) {
        if ($PaymentMethodItem->get('Class')->alias eq 'PaymentMethodMolliePayments') {
            $MollieExists = 1;
            last;
        }
    }

    return $MollieExists;
}

sub MollieDisplayIconsAlways {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($System) = @$aParams;
    my @PaymentIcons;

    if ($System->get("IsMolliePaymentStandard")) {
        foreach my $PaymentIcon (@DISPLAY_PAYMENT_ICONS) {
            if ($PaymentIcon->{'DisplayAlways'}) {
                push(@PaymentIcons, {   'ID'   => $PaymentIcon->{'ID'},
                    'Icon' => $PaymentIcon->{'Icon'},
                    'Name' => $PaymentIcon->{'Name'} 
                });
            }
        }
    }
    return \@PaymentIcons;
}

sub MollieLimitedPaymentMethods {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($System) = @$aParams;
    my @LimitedPaymentMethods;

    if ($System->get("IsMolliePaymentStandard")) {
        foreach my $LimitedPaymentMethod (@MOLLIE_LIMITED_PAYMENTMETHODS) {
            push(@LimitedPaymentMethods, {   'ID'   => $LimitedPaymentMethod->{'ID'},
                'Icon' => $LimitedPaymentMethod->{'Icon'},
                'Name' => $LimitedPaymentMethod->{'Name'} 
            });
        }
    }
    return \@LimitedPaymentMethods;
}

sub MollieDisplayLocalizedIcons {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Shop) = @$aParams;
    my @LocalizedPaymentIcons;

    my $System = LoadRootObject();
    if ($System->get("IsMolliePaymentStandard")) {
        foreach my $PaymentIcon (@DISPLAY_PAYMENT_ICONS) {
            if ($PaymentIcon->{'LocaleID'} eq $Shop->get('LocaleID')) {
                push(@LocalizedPaymentIcons, {  'ID'   => $PaymentIcon->{'ID'},
                    'Icon' => $PaymentIcon->{'Icon'},
                    'Name' => $PaymentIcon->{'Name'} 
                });
            }
        }
    }
    return \@LocalizedPaymentIcons;
}

1;
