#==============================================================================
# package      DE_EPAGES::MolliePayments::Hooks::MolliePayment
# state        private
#==============================================================================
package DE_EPAGES::MolliePayments::Hooks::MolliePayments;

use strict;
use DE_EPAGES::MolliePayments::API::Constants qw (
    @SPLIT_GATEWAY_METHODS
    @MOLLIE_LIMITED_PAYMENTMETHODS
    $SPLIT_GATEWAY_METHODS_MAXROWS
    PAYMENT_TYPE_MOLLIE_PAYMENTS
);
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::MolliePayments::API::Requests::Methods;
use DE_EPAGES::MolliePayments::API::MolliePaymentsClient;


#========================================================================================
# function
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnViewMultiMolliePayment($hParams);
#----------------------------------------------------------------------------------------
# description  UI_UnityMBOViewPaymentList triggered, send requests to Mollie API
#               and prefill needed Servlet variables
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnViewMultiMolliePayment{
    my ($hParams) = @_;
    my $System = LoadRootObject();

    return unless $System->get('IsMolliePaymentStandard');

    my $Servlet = $hParams->{'Servlet'};
    my $Shop = $Servlet->object;
    my $PaymentMethodClassName = 'PaymentMethodMolliePayments';
    my $MethodAttributeName = 'Method';

    my $PaymentMethods = $Shop->folder('PaymentMethods')->children;
    my $MolliePaymentMethod;
    my $hSavedSettings;

    foreach my $PaymentMethodItem (@$PaymentMethods) {
        if ($PaymentMethodItem->get('Class')->alias eq 'PaymentMethodMolliePayments') {
            $MolliePaymentMethod = $PaymentMethodItem;
            $hSavedSettings = $PaymentMethodItem->get([qw(ProfileId IsInTestMode Method)]);
            last;
        }
    }

    return unless defined $MolliePaymentMethod;

    my $hPaymentMethods = _getPaymentMethodsList($Servlet, $MolliePaymentMethod, $hSavedSettings);
    $Servlet->vars( 'MethodsList', $hPaymentMethods );

    my @EnabledMethodsId = map { $_->{'id'} } @{$hPaymentMethods->{'methods'}};
    my $TranslatedPaymentMethodNames = $hPaymentMethods->{'methods'};

    # Adapt the constant payment method names with the payment method names from the api call
    foreach my $Payment (@{$TranslatedPaymentMethodNames}) {
        foreach my $Gateway (@SPLIT_GATEWAY_METHODS) {
            if ($Gateway->{'ID'} eq $Payment->{'id'}) {
                $Gateway->{'Name'} = $Payment->{'description'};
            }
        }
    }

        my $aAllValidMollieMethods = $Shop->featureMaxValue('MollieLimited') ? \@MOLLIE_LIMITED_PAYMENTMETHODS : \@SPLIT_GATEWAY_METHODS;
        $Servlet->vars('SplitGatewayMethods_MaxRows',$SPLIT_GATEWAY_METHODS_MAXROWS);
        $Servlet->vars('SplitGatewayMethods', $aAllValidMollieMethods );
        $Servlet->vars('SplitGatewayMethods_DialogOneColumn', 1);

        if (isValid(\@EnabledMethodsId)) {
            _filterAndSetBatchPaymentMethods($Servlet, $aAllValidMollieMethods, \@EnabledMethodsId);
        }

    $Servlet->vars('CanReplicatePayment', $MolliePaymentMethod->usableAtStorefront());

    foreach my $PaymentMethodItem (@$PaymentMethods) {
        next unless $PaymentMethodItem->class()->alias() eq $PaymentMethodClassName;
        my $Method = $PaymentMethodItem->get($MethodAttributeName);
        next unless isValid($Method);
        $Servlet->vars('ExistingMultiPaymentMethod_' . $PaymentMethodItem->get($MethodAttributeName), 1);
    }

    return;
}

#----------------------------------------------------------------------------------------
sub _getPaymentMethodsList {
    my ($Servlet, $PaymentMethod, $hSavedSettings) = @_;

    my $IsInTestMode = $hSavedSettings->{'IsInTestMode'};
    my $ProfileId = $hSavedSettings->{'ProfileId'};

    my $hMethodsData = {
        'payment' => $PaymentMethod
    };

    my $MethodsHandler = _createMethodsHandler($hMethodsData);
    $MethodsHandler->withProfileId($ProfileId);
    my $MollieService = _createMolliePaymentsClient($MethodsHandler);

    my $hLogParams = {
        'PaymentMethodId' => $PaymentMethod->id(),
        'profileId' => $ProfileId,
    };
    my $hMethodsResponse = {};

    eval {
        $hMethodsResponse = $MollieService->withTestMode($IsInTestMode)->doGetRequest();
    };
    if (ExistsError()) {
        my $Error = GetError();
        $hLogParams->{'Error'} = $Error->message();
        _LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS,
            'Error when getting Payment Methods', $hLogParams, undef, undef
        );
        $Servlet->vars('MollieConnectionError', 1);
        return {};
    }

    $hLogParams->{'MethodsResponse'} = $hMethodsResponse;
    _LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'Get Payment Methods response', $hLogParams, undef , undef);

    return $hMethodsResponse;
}
#----------------------------------------------------------------------------------------
sub _createMethodsHandler {
    my ($hMethodsData) = @_;

    return DE_EPAGES::MolliePayments::API::Requests::Methods->new($hMethodsData);
}
#----------------------------------------------------------------------------------------
sub _createMolliePaymentsClient {
    my ($HandlerObject) = @_;

    return DE_EPAGES::MolliePayments::API::MolliePaymentsClient->new($HandlerObject);
}
#========================================================================================
sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, $Name, $DebugData, $Section, $Filter);
}
#----------------------------------------------------------------------------------------
sub _filterAndSetBatchPaymentMethods {
    my ($Servlet, $aAllValidMollieMethods, $aFilterMethodsId) = @_;

    my @FilteredMethods;
    my $CountAllMethods = scalar(@{$aAllValidMollieMethods});
    my $CountGroupMethods = 0;
    my $MaxMethodsRows = 0;

    for (my $i = $CountAllMethods - 1; $i >= 0; $i--) {

        if ($aAllValidMollieMethods->[$i]->{'isGroup'}) {
            if ($CountGroupMethods) {
                $aAllValidMollieMethods->[$i]->{'GrpCount'} = $CountGroupMethods;
                push(@FilteredMethods, $aAllValidMollieMethods->[$i]);
                $MaxMethodsRows = ($CountGroupMethods > $MaxMethodsRows) ? $CountGroupMethods : $MaxMethodsRows;
            }
            $CountGroupMethods = 0;
            next;
        }

        if ($aAllValidMollieMethods->[$i]->{'ID'} ~~ @{$aFilterMethodsId}) {
            push(@FilteredMethods, $aAllValidMollieMethods->[$i]);
            $CountGroupMethods++;
        }
    }

    $Servlet->vars('SplitGatewayMethods_MaxRows', $MaxMethodsRows);
    $Servlet->vars('SplitGatewayMethods', [ reverse(@FilteredMethods) ]);
}
#----------------------------------------------------------------------------------------

1;
