#========================================================================================
# package      DE_EPAGES::Order::API::BasketLineItemContainer
# base         Base
# state        public
# description  Creates and sets the LineItemContainer of a basket.
#========================================================================================
package DE_EPAGES::Order::API::BasketLineItemContainer;

use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByGUID LoadObjectByGUID );
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode );
use DE_EPAGES::Shop::API::Constants qw ( Text2TaxModel );
use DE_EPAGES::Order::API::LineItemContainer qw ( SaveShipping SavePayment );
use DE_EPAGES::Presentation::API::Form;
use DE_EPAGES::Order::API::ProductValidator qw ( ValidateStockLevelSettings ValidateAvailibilityVisibilitySettings );

use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Basket );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# input        $Basket | basket | DE_EPAGES::Order::API::Object::Basket
# return       $self | self | DE_EPAGES::Order::API::BasketLineItemContainer
#========================================================================================
sub new {
    my $class = shift;
    my ($Basket) = @_;
    my $self  = bless {
        'Basket' => $Basket,
    }, $class;
    return $self;
}



#========================================================================================
# function     setLineItemContainer
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->setLineItemContainer( $hContainer )
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Create a new LineItemContainer if not exists and set its content.
#----------------------------------------------------------------------------------------
# input        $hContainer | data of line item container as ref hash:
#                             <ul>
#                               <li>TaxModel       - string (net or gross)
#                               <li>LocaleID       - Char(5)
#                               <li>LanguageCode   - Char(3)
#                               <li>CurrencyID     - Char(3)
#                               <li>ProductLineItems - array of product line items
#                                 <ul>
#                                   <li>Product    - string (GUID of object)
#                                   <li>Quantity   - float
#                                 </ul> - ref.array.hash
#                               <li>PaymentMethod  - string (path to object)
#                               <li>ShippingMethod - string (path to object)
#                             </ul> | ref.hash
#========================================================================================
sub setLineItemContainer {
    my $self = shift;
    my ($hContainerData, $hSellingCountryData)  = @_;

    return unless $hContainerData;

    my $Basket = $self->{'Basket'};
    my $LineItemContainer = $Basket->container();

    my $Shop = $Basket->getSite();
    my $LanguageID = $hContainerData->{'LanguageCode'}
        ? GetPKeyLanguageByCode($hContainerData->{'LanguageCode'})
        : $Shop->get('LanguageID');
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $LanguageID;

    if ($hContainerData->{'LocaleID'}) {
        $LineItemContainer->set({'LocaleID' => $hContainerData->{'LocaleID'}});
    }
    if($hContainerData->{'CurrencyID'}) {
        $LineItemContainer->set({'CurrencyID' => $hContainerData->{'CurrencyID'}});
    }
    if($hContainerData->{'TaxModel'}) {
       $LineItemContainer->set({'TaxModel' => Text2TaxModel($hContainerData->{'TaxModel'})});
    }

    $self->_setProductLineItems($LineItemContainer, $hContainerData);

    $self->_setShippingOptions($LineItemContainer, $hContainerData);

    if ( $hContainerData->{'PaymentMethod'} ) {
        my $PaymentMethod = $self->loadShopObject($hContainerData->{'PaymentMethod'}, 'PaymentMethod');
        SavePayment( $LineItemContainer, $PaymentMethod );
    }

    if ( $hContainerData->{'ShippingMethod'} ) {
        my $ShippingMethod = $self->loadShopObject($hContainerData->{'ShippingMethod'}, 'ShippingMethod');
        SaveShipping( $LineItemContainer, $ShippingMethod );
    }

    # we have an active delivery threshold, use the correct TaxMatrix
    if ($Shop->get('DeliveryThresholdIsActive')){
        my $hContainerInfo = $LineItemContainer->get([qw(TaxArea TaxMatrix)]);
        # set correct TaxMatrix on the container by changing the TaxArea
        if($hContainerInfo->{'TaxMatrix'}->id != $hSellingCountryData->{'TaxMatrix'}->id && $hSellingCountryData->{'IsEU'}){
            $LineItemContainer->set({'TaxArea' => $hSellingCountryData->{'TaxMatrix'}->child($hContainerInfo->{'TaxArea'}->alias)});
        } else {
            # in this case we are switching from eu to non eu and vice versa
            my $TaxArea = $hContainerInfo->{'TaxMatrix'}->taxAreaByCountry($hSellingCountryData->{'SellingCountryID'});
            $LineItemContainer->set({'TaxArea' => $TaxArea});
        }

        if ($Shop->get('TaxModel') == 1) {
            $LineItemContainer->set({'TaxModel' => $hSellingCountryData->{'TaxModel'}});
        }
    }

    $LineItemContainer->recalculate();
    return;
}



#========================================================================================
# function     _setProductLineItems
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setProductLineItems( $LineItemContainer, $hContainerData );
#----------------------------------------------------------------------------------------
# description  Sets the product line items in the LineItemContainer
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# input        $hContainerData | data for container | ref_hash
#========================================================================================
sub _setProductLineItems {
    my $self = shift;
    my ($LineItemContainer, $hContainerData) = @_;

    my $Basket = $self->{'Basket'};
    my $Shop = $Basket->getSite();

    foreach my $hLineItem (@{$hContainerData->{'ProductLineItems'}}) {

        my $Product = $self->_loadShopObjectByGUID($Shop, $hLineItem->{'Product'}, 'Product');
        ValidateStockLevelSettings($Product, $hLineItem->{'Quantity'});
        ValidateAvailibilityVisibilitySettings($Product);

        if ( ($hLineItem->{'Quantity'} eq '0') || ($hLineItem->{'Quantity'} == 0) ) {
            my $LineItems = $LineItemContainer->get('LineItems');
            my @SameProductLineItems =
                grep { $_->instanceOf('LineItemProduct') and
                       defined $_->get('Product') and
                       $_->get('Product')->id() == $Product->id()
                      } @$LineItems;
            scalar( @SameProductLineItems) ? $SameProductLineItems[0]->delete : undef;
        } else {
            $Product->addToContainer($LineItemContainer, {
                'Quantity' => $hLineItem->{'Quantity'} || 1,
            });
        }
    }

    return;
}

#========================================================================================
# function     _setShippingOptions
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setShippingOptions( $LineItemContainer, $hContainerData );
#----------------------------------------------------------------------------------------
# description  Sets the shipping options in the LineItemContainer
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# input        $hContainerData | data for container | ref_hash
#========================================================================================
sub _setShippingOptions {
    my $self = shift;
    my ($LineItemContainer, $hContainerData) = @_;

    foreach my $hLineItem (@{$hContainerData->{'ShippingOptionLineItems'}}) {
        my $ShippingOption = $self->loadShopObject($hLineItem->{'ShippingOption'}, 'ShippingOption');
        $ShippingOption->addToContainer($LineItemContainer, {
            'Quantity' => $hLineItem->{'Quantity'} || 1,
        });
    }

    return;
}



#========================================================================================
# function     loadShopObjectByGUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $self->loadShopObjectByGUID( $GUID, $ClassAlias )
#----------------------------------------------------------------------------------------
# description  loads object specified by object GUID and verifies that is exists,
#               belongs to the current shop and is an instance of a certain class.
#               Raises an error if any condition is not fulfilled.
#               Returns the object if all tests passed.
#----------------------------------------------------------------------------------------
# input        $Shop | a shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $GUID | object GUID to load | string
# input        $ClassAlias | alias of a class | string
# return       $Object| object | object
#========================================================================================
sub _loadShopObjectByGUID {
    my $self = shift;
    my ($Shop, $GUID, $ClassAlias) = @_;

    Error('OBJECT_NOT_FOUND', {'GUID' => $GUID}) unless ExistsObjectByGUID($GUID);

    my $ObjectID = LoadObjectByGUID($GUID)->id();
    return $Shop->getSite()->loadObject($ObjectID, $ClassAlias // 'Object');
}


1;
