#========================================================================================
# package      DE_EPAGES::Order::API::LineItemContainerData
# state        public
#----------------------------------------------------------------------------------------
# description  Gets the data of LineItemContainer and its line items.
#========================================================================================
package DE_EPAGES::Order::API::LineItemContainerData;

use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_GROSS TAX_MODEL_NET );
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Shop::API::Resource::Formatter;
use DE_EPAGES::Order::API::LineItemsData;
use DE_EPAGES::Order::API::OtherLineItemsData;
use DE_EPAGES::Order::API::SplitableTaxes;
use List::Util qw( reduce );

use strict;


#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer object | object
# return       $self | self | DE_EPAGES::Order::API::LineItemContainerData
#========================================================================================
sub new {
    my $class = shift;
    my ($LineItemContainer)  = @_;

    TestObject('DE_EPAGES::Order::API::Object::LineItemContainer', $LineItemContainer) if defined($LineItemContainer);
    my $self  = bless {
        'LineItemContainer' => $LineItemContainer,
    }, $class;
    return $self;
}

sub getCurrencyOfLineItemContainer{
    my $self = shift;
    my ($LanguageID) = @_;

    my $LineItemContainer = $self->{'LineItemContainer'};
    return undef unless $LineItemContainer;

    return $LineItemContainer->get('CurrencyID');
}

sub getLineItemContainerData {
    my $self = shift;
    my ($LanguageID, $LocaleID, $IncludeTaxInLineItem, $hSellingCountryData) = @_;

    my $LineItemContainer = $self->{'LineItemContainer'};
    return undef unless $LineItemContainer;

    if (defined $hSellingCountryData and $LineItemContainer->getSite()->get('DeliveryThresholdIsActive')){
        my $ShippingCountryID = $LineItemContainer->get('ShippingCountryID');

        # change Taxmatrix to shipping destination if we have no shipping address already on the basket
        unless ($ShippingCountryID) {
            my $hContainerInfo = $LineItemContainer->get([qw(TaxArea TaxMatrix)]);
            # set correct TaxMatrix on the container by changing the TaxArea
            if($hContainerInfo->{'TaxMatrix'}->id != $hSellingCountryData->{'TaxMatrix'}->id && $hSellingCountryData->{'IsEU'}){
                $LineItemContainer->set({'TaxArea' => $hSellingCountryData->{'TaxMatrix'}->child($hContainerInfo->{'TaxArea'}->alias)});
            } else {
                # in this case we are switching from eu to non eu and vice versa
                my $TaxArea = $hContainerInfo->{'TaxMatrix'}->taxAreaByCountry($hSellingCountryData->{'SellingCountryID'});
                $LineItemContainer->set({'TaxArea' => $TaxArea});
            }

            if ($LineItemContainer->getSite()->get('TaxModel') == 1) {
                $LineItemContainer->set({'TaxModel' => $hSellingCountryData->{'TaxModel'}});
            }

            $LineItemContainer->recalculate;

        }
    }

    my @ContainerAttributes = qw(CurrencyID TaxModel GrandTotal TotalBeforeTax TotalTax LineItemsSubTotal);
    my $hContainer = $LineItemContainer->get(\@ContainerAttributes, $LanguageID);
    my ($CurrencyID, $TaxModel) = @{$hContainer}{qw(CurrencyID TaxModel)};
    my $Formatter = $self->_createFormatter($LineItemContainer->getSite(), $LocaleID);

    my $LineItemShipping = $LineItemContainer->get('Shipping');
    my $ShippingPrice = defined($LineItemShipping) ? $LineItemShipping->get('LineItemPrice') : 0;

    my $OtherLineItems = $self->_otherLineItems($LineItemContainer, $Formatter, $LanguageID, $IncludeTaxInLineItem);
    return {
        productLineItems => $self->_lineItems($LineItemContainer, $Formatter, $LanguageID, $IncludeTaxInLineItem),
        couponLineItem => $self->getCouponData($CurrencyID, $Formatter),
        grandTotal => $Formatter->price($hContainer->{'GrandTotal'}, $CurrencyID, TAX_MODEL_GROSS),
        totalBeforeTax => $Formatter->price($hContainer->{'TotalBeforeTax'}, $CurrencyID, TAX_MODEL_NET),
        totalTax => $Formatter->price($self->getTotalTaxAmount($hContainer->{'TotalTax'}), $CurrencyID, undef),
        lineItemsSubTotal => $Formatter->price($hContainer->{'LineItemsSubTotal'}, $CurrencyID, $TaxModel),
        shippingPrice => $Formatter->price($ShippingPrice, $CurrencyID, $TaxModel),
        otherLineItems => $OtherLineItems,
    };
}

sub getCouponData {
    my $self = shift;
    my ($CurrencyID, $Formatter) = @_;
    my $LineItemContainer = $self->{'LineItemContainer'};
    return undef unless $LineItemContainer;

    my $LineItemCoupon = $LineItemContainer->get('Coupon');
    if(!defined($LineItemCoupon)){
        return undef;
    }

    my $CouponCampaignID = $self->_getCouponCampaignID($LineItemCoupon);

    my @ContainerAttributes = qw(LineItemPrice GUID ValidationErrors);
    my $hInLineItem = $LineItemCoupon->get(\@ContainerAttributes);

    my @ValidationErrors = map{ $_->{'Message'} } @{$hInLineItem->{'ValidationErrors'}};

    my $hCouponData = {
        'couponLineItemId' => $hInLineItem->{'GUID'},
        'couponCampaignId' => $CouponCampaignID ? $CouponCampaignID : 'NotFound',
        'lineItemPrice' => $Formatter->price($hInLineItem->{'LineItemPrice'}, $CurrencyID, undef),
        'validationErrors' => \@ValidationErrors
    };
    return $hCouponData;
}

sub getTotalTaxAmount {
    my $self = shift;
    my ($TotalTax) = @_;

    return $TotalTax unless ($TotalTax == 0);

    # take into account LineItemPrice value from LineItemTax when it's negative
    # because it means order taxation has been changed in MBO (AD-10254)
    my $LineItemTax;
    foreach my $LineItem ( @{$self->{'LineItemContainer'}->children()} ) {
        if ( $LineItem->instanceOf('LineItemTax') ) {
            $LineItemTax = $LineItem;
            last;
        }
    }
    return $TotalTax unless $LineItemTax;

    my $LineItemPrice = $LineItemTax->get('LineItemPrice');

    if ($LineItemPrice < 0) {
        return $LineItemPrice;
    } else {
        return $TotalTax;
    }
}

sub getShippingData {
    my $self = shift;
    my ($LanguageID, $LocaleID, $IncludeTaxInLineItem) = @_;

    my $LineItemContainer = $self->{'LineItemContainer'};
    return undef unless $LineItemContainer;

    my $LineItemShipping = $LineItemContainer->get('Shipping');
    return undef unless $LineItemShipping;

    my @ContainerAttributes = qw(CurrencyID TaxModel GrandTotal TotalBeforeTax TotalTax LineItemsSubTotal);
    my $hContainer = $LineItemContainer->get(\@ContainerAttributes, $LanguageID);
    my ($CurrencyID, $TaxModel) = @{$hContainer}{qw(CurrencyID TaxModel)};

    my $ShippingMethod = $LineItemShipping->get('ShippingMethod');
    my $ShippingPrice = $LineItemShipping->get('LineItemPrice');

    my $hShippingMethod;
    if(defined $ShippingMethod) {
        $hShippingMethod = {
            'id' => $ShippingMethod->guid(),
            'name' => $ShippingMethod->get('NameOrAlias', $LanguageID),
        };
    }
    my $Formatter = $self->_createFormatter($LineItemContainer->getSite(), $LocaleID);
    my $hShippingData = {
        'shippingMethod' => $hShippingMethod,
        'price' => $Formatter->price($ShippingPrice, $CurrencyID, $TaxModel),
    };
    # TaxArea could be unavailable (AD-10419)
    my $aTaxes;
    if ($self->_canCalculateTaxes($LineItemContainer, $LineItemShipping)) {
        $aTaxes = $self->_calculateShippingTaxes($LineItemContainer, $LanguageID, $Formatter);
    } else {
        GetLog->debug(
            sprintf('Shipping tax can not be calculated for %s', $LineItemContainer->parent()->get('Path'))
        );
        $aTaxes = $self->_getLineItemTaxes($LineItemContainer, $LineItemShipping, $LanguageID, $Formatter);
    }
    $hShippingData->{'taxes'} = $aTaxes;

    if (defined($IncludeTaxInLineItem) && $IncludeTaxInLineItem eq 'false' && $TaxModel == TAX_MODEL_GROSS) {
        my $ShippingTax = $self->_calculateTotalTax($aTaxes);
        my $ShippingNetPrice = $ShippingPrice - $ShippingTax;
        $hShippingData->{'price'} = $Formatter->price($ShippingNetPrice, $CurrencyID, TAX_MODEL_NET);
    }

    return $hShippingData;
}

sub _getCouponCampaignID {
    my $self = shift;
    my ($LineItemCoupon) = @_;

    my $CouponCode = $LineItemCoupon->get('CouponCode');
    return unless($CouponCode);
    my $CouponCampaign = $CouponCode->get('CouponCampaign');

    return $CouponCampaign->guid();
}

sub _calculateTotalTax{
    my $self = shift;
    my ($aTaxes) = @_;
    my $TaxTotalAmount = 0;
    my @AmountPrices = map { $_->{'price'}->{'amount'} } @{$aTaxes};
    $TaxTotalAmount = reduce { $a + $b } @AmountPrices;
    return $TaxTotalAmount;
}

sub _calculateShippingTaxes{
    my $self = shift;
    my ($LineItemContainer, $LanguageID, $Formatter) = @_;

    my $SplitableTaxes = DE_EPAGES::Order::API::SplitableTaxes->new();
    return $SplitableTaxes->calculateShippingTax($LineItemContainer, $LanguageID, $Formatter);
}

sub _calculatePaymentTaxes{
    my $self = shift;
    my ($LineItemContainer, $LanguageID, $Formatter) = @_;

    my $SplitableTaxes = DE_EPAGES::Order::API::SplitableTaxes->new();
    return $SplitableTaxes->calculatePaymentTax($LineItemContainer, $LanguageID, $Formatter);
}

sub _canCalculateTaxes {
    my $self = shift;
    my ($LineItemContainer, $LineItem) = @_;

    my $hNetPriceForTax = $LineItemContainer->netPricesForTax();
    my $hLineItemPriceByTaxClassId;
    if ($LineItem->instanceOf('LineItemShipping')) {
        $hLineItemPriceByTaxClassId = $hNetPriceForTax->{'LineItemShipping'};
    } elsif ($LineItem->instanceOf('LineItemPayment')) {
        $hLineItemPriceByTaxClassId = $hNetPriceForTax->{'LineItem'};
    }
    return 0 unless (defined $hLineItemPriceByTaxClassId);

    my $TaxArea = $LineItemContainer->get('TaxArea');
    my $TaxAreaDigital = $LineItemContainer->get('TaxAreaDigital');
    foreach my $TaxClassId (keys %$hLineItemPriceByTaxClassId){
        my $TaxClass = LoadObject($TaxClassId);
        return 0 unless (defined $TaxClass);

        if ($TaxClass->instanceOf('TaxClassDigital')) {
            return 0 unless (defined $TaxAreaDigital);
        } elsif ($TaxClass->instanceOf('TaxClass')) {
            return 0 unless (defined $TaxArea);
        }
    }
    return 1;
}

sub _getLineItemTaxes {
    my $self = shift;
    my ($LineItemContainer, $LineItem, $LanguageID, $Formatter) = @_;

    return [] unless(defined $LineItem);
    my @Taxes = ();
    my $CurrencyID = $LineItemContainer->get('CurrencyID');
    my $aLineItemTaxes = $LineItem->get('Taxes');
    foreach my $hTaxInfo (@$aLineItemTaxes) {
        my $TaxClass = $hTaxInfo->{'TaxClass'};
        my $hTax = {
            price => $Formatter->price($hTaxInfo->{'TaxAmount'}, $CurrencyID, undef),
            taxClass => {
                name => $TaxClass->get('Name', $LanguageID),
                percentage => $hTaxInfo->{'TaxRate'} * 100,
                taxClassId => $TaxClass->guid()
            }
        };
        push(@Taxes, $hTax);
    }
    return \@Taxes;
}

sub getCartValidations {
    my $self = shift;
    my ($LanguageID) = @_;

    my $LineItemContainer = $self->{'LineItemContainer'};
    return undef unless $LineItemContainer;

    # add possible shipping methods to cartValidation
    my $aShippingMethods = $LineItemContainer->get('ShippingMethods');
    return undef unless $aShippingMethods;
    my $hFields = {};
    my $aPossible = [];
    foreach my $ShippingMethod (@$aShippingMethods) {
        push @{$aPossible}, $ShippingMethod->get('NameOrAlias', $LanguageID);
    }
    $hFields->{'possibleShippingMethods'} = $aPossible;

    return $hFields;
}

sub getPaymentData {
    my $self = shift;
    my ($LanguageID, $LocaleID, $IncludeTaxInLineItem) = @_;

    my $LineItemContainer = $self->{'LineItemContainer'};
    return undef unless $LineItemContainer;

    my $LineItemPayment = $LineItemContainer->get('Payment');
    return undef unless $LineItemPayment;

    my @ContainerAttributes = qw(CurrencyID TaxModel GrandTotal TotalBeforeTax TotalTax LineItemsSubTotal);
    my $hContainer = $LineItemContainer->get(\@ContainerAttributes, $LanguageID);
    my ($CurrencyID, $TaxModel) = @{$hContainer}{qw(CurrencyID TaxModel)};

    my $PaymentMethod = $LineItemPayment->get('PaymentMethod');
    my $PaymentPrice = $LineItemContainer->get('Payment')->get('LineItemPrice') // 0;
    if(defined $LineItemContainer->get('PaymentDiscount')) { # payment price/fee is store on the lineItemPaymentDiscount
       $PaymentPrice += $LineItemContainer->get('PaymentDiscount')->get('LineItemPrice');
    }

    my $IsExternalPayment = $LineItemPayment->instanceOf('LineItemPaymentExternal');

    my $hPaymentMethod;
    if (defined $PaymentMethod) {
        my $ProviderName;
        my $AdditionalData;
        if ($IsExternalPayment) {
            $ProviderName = $PaymentMethod->getProviderName($LanguageID);
            $AdditionalData = $LineItemPayment->getAdditionalData();
        }
        $hPaymentMethod = {
            'id'             => $PaymentMethod->guid(),
            'name'           => $PaymentMethod->get('NameOrAlias', $LanguageID),
            'providerName'   => $ProviderName,
            'additionalData' => $AdditionalData
        };
    }

    my $TransactionId = $IsExternalPayment ? $LineItemPayment->getDisplayTransID() : undef;

    my $HasPaymentErrorCode = $LineItemPayment->class()->existsAttribute('PaymentErrorCode');
    my $PaymentErrorCode = $HasPaymentErrorCode ? $LineItemPayment->get('PaymentErrorCode') : undef;

    my $Formatter = $self->_createFormatter($LineItemContainer->getSite(), $LocaleID);
    my $hPaymentData = {
        'paymentMethod' => $hPaymentMethod,
        'transactionId' => $TransactionId,
        'price' => $Formatter->price($PaymentPrice, $CurrencyID, $TaxModel),
        'status' => $PaymentErrorCode,
    };
    # TaxArea could be unavailable (AD-10419)
    my $aTaxes;
    if ($self->_canCalculateTaxes($LineItemContainer, $LineItemPayment)) {
        $aTaxes = $self->_calculatePaymentTaxes($LineItemContainer, $LanguageID, $Formatter);
    } else {
        GetLog->debug(
            sprintf('Payment tax can not be calculated for %s', $LineItemContainer->parent()->get('Path'))
        );
        $aTaxes = $self->_getLineItemTaxes($LineItemContainer, $LineItemPayment, $LanguageID, $Formatter);
    }
    $hPaymentData->{'taxes'} = $aTaxes;

    if (isValid($IncludeTaxInLineItem) && $IncludeTaxInLineItem eq 'false' && $TaxModel == TAX_MODEL_GROSS) {
        my $PaymentTax = $self->_calculateTotalTax($aTaxes);
        my $PaymentNetPrice = $PaymentPrice - $PaymentTax;
        $hPaymentData->{'price'} = $Formatter->price($PaymentNetPrice, $CurrencyID, TAX_MODEL_NET);
    }
    return $hPaymentData;
}

sub _lineItems {
    my $self = shift;
    my ($Container, $Formatter, $LanguageID, $IncludeTaxInLineItem) = @_;

    my $LineItemsData = $self->_createLineItemsData();
    return $LineItemsData->getLineItems($Container, $Formatter, $LanguageID, $IncludeTaxInLineItem);
}


sub _otherLineItems {
    my $self = shift;
    my ($Container, $Formatter, $LanguageID, $IncludeTaxInLineItem) = @_;

    my $LineItemsData = $self->_otherLineItemsData();
    return $LineItemsData->getOtherLineItems($Container, $Formatter, $LanguageID, $IncludeTaxInLineItem);
}

sub _otherLineItemsData {
    my $self = shift;
    return DE_EPAGES::Order::API::OtherLineItemsData->new();
}
sub _createLineItemsData {
    my $self = shift;
    return DE_EPAGES::Order::API::LineItemsData->new();
}

sub _createFormatter {
    my $self = shift;
    my ($Shop, $LocaleID) = @_;
    return DE_EPAGES::Shop::API::Resource::Formatter->new($Shop, $LocaleID);
}

1;
