#==============================================================================
# package      DE_EPAGES::PayPalExpress::UI::Basket
# state        private
#==============================================================================
package DE_EPAGES::PayPalExpress::UI::Basket;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Object::API::Table::Country qw (
    ExistsCountryByCode2
    GetPKeyCountryByCode2
);
use DE_EPAGES::Order::API::LineItemContainer qw (
    CheckSaveShipping
    SavePayment
    UpdateTaxModel
);
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::PayPalExpress::API::Constants qw ( API_VERSION );
use DE_EPAGES::PayPalExpress::API::Payment qw ( GetInstantUpdateParams ResetBasket );
use DE_EPAGES::PayPalExpress::API::Webservice::BasketService;
use DE_EPAGES::PayPalExpress::API::Webservice::GetExpressCheckoutDetails;
use URI::Escape qw ( uri_escape_utf8 );


#========================================================================================
# function     ViewPayPalExpressFailed
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewPayPalExpressFailed();
#----------------------------------------------------------------------------------------
# description  UI Action to view default error (Base) or reassign to Unity (now).
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub ViewPayPalExpressFailed {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object();
    my $Shop = $Basket->getSite();
    if ($Shop->isUnity()) {
        $Servlet->vars('ViewAction', 'UnityViewSFPayPalExpressFailed');
    }

    $self->SUPER::View($Servlet);

    return;
}

#==============================================================================
sub PayPalExpress {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;

    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'IsBlocked' => 1,
            'OrderGUID' => $Basket->get('OrderGUID')
        },
        'ViewAction' => 'ViewBlocked'
    }) if (defined($Basket->get('OrderGUID')));

    return unless defined (my $Container = $Basket->container);

    my $NewPaymentMethod = undef;
    #--- PaymentMethodID sent as parameter?
    if ($Servlet->form->value('PaymentMethodID')) {
        eval {
            $NewPaymentMethod = $Basket->getSite()->loadObject($Servlet->form->value('PaymentMethodID'), 'PaymentMethodPayPalExpress');
        };
        undef($NewPaymentMethod) if ExistsError();
    }
    #--- PaymentMethod not found - take first PaymentMethod of Type PayPalExpress
    if (!defined($NewPaymentMethod)) {
        my $aPaymentMethods = $Container->get('PaymentMethods');
        foreach my $PaymentMethod (@$aPaymentMethods) {
            if ($PaymentMethod->instanceOf('PaymentMethodPayPalExpress')) {
                $NewPaymentMethod = $PaymentMethod;
                last;
            }
        }
    }
    Error('PayPalExpressNotAvailabe', {'BasketID' => $Basket->id}) unless defined $NewPaymentMethod;

    #--- change payment method
    SavePayment($Container, $NewPaymentMethod);
    $Container->recalculate;

    #--- reset IsChanged attribute (Bug EPG-4624)
    foreach my $LineItem (@{$Container->get('LineItems')}) {
        $LineItem->set({'IsChanged' => 0});
    }

    # EPG-13060: check if payment line item is defined and is of type PayPalExpress
    my $pli = $Container->get('Payment');
    unless(defined($pli) && $pli->instanceOf('LineItemPaymentPayPalExpress')) {
        LogPayment('PayPalExpress', $pli->id . ' is not of instance LineItemPaymentPayPalExpress', {});
        Error('INVALID_FORM', {
            'ViewVars'=> {
                'FormError' => 0
            },
            'ViewAction' => 'ViewPayPalExpressFailed'
        });
    }

    #--- start payment processing
    my $redirectUrl = $pli->checkoutRequest($Servlet, 1);

    die 'no redirectUrl' unless $redirectUrl;

    $Servlet->vars('ObjectID', $Basket->id);

    #--- init a cookie that we're redirecting to PP
    $Servlet->redirect($redirectUrl);

    return;
}
#========================================================================================
# function     _logRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       _logRequest( $Servlet );
#----------------------------------------------------------------------------------------
# description  logs the shop alias and the parameters of the call
#----------------------------------------------------------------------------------------
# input        $Servlet | epages servlet  | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub _logRequest {
    my $self = shift;
    my ( $Servlet ) = @_;

    eval {
        my $ShopAlias = $Servlet->object()->getSite()->alias();
        my $hLogParameters = {
            'ShopAlias' => $ShopAlias,
            'RequestParameters' => $Servlet->params(),
        };
        LogPayment( 'PayPalExpress', 'PayPalExpressReturn incomming call', $hLogParameters );
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        LogError( $Error );
    }

    return;
}
#========================================================================================
# function     PayPalExpressReturn
# state        public
#----------------------------------------------------------------------------------------
# syntax       PayPalExpressReturn( $Servlet );
#----------------------------------------------------------------------------------------
# description  Change action for pay pal express checkout after the customer paid at pay pal
#               returns <i>undef</i> if action was called with a paypalexpressCheckout single
#               click button order process.
#               returns an <i>Order</i> if the action was called with a normal epages checkout process
#----------------------------------------------------------------------------------------
# input        $Servlet | epages servlet  | DE_EPAGES::Presentation::Servlet::Permission
# return       $ReturnValue | undef or Order | DE_EPAGES::Order::API::Object::CustomerOrder
#========================================================================================
sub PayPalExpressReturn {
    my $self     = shift;
    my ( $Servlet ) = @_;

    $self->_logRequest( $Servlet );

    my $Basket = $Servlet->object();

    my $Shop = $Basket->get('Shop');
    if($Shop->get('IsUnityRegistered')){
        my $UserIdentifiedPerCookie = 0;
        $Servlet->registerSession($Basket->get('User'), $UserIdentifiedPerCookie);
    }

    return unless $self->_IsAllowedRequest( $Servlet );

    my $BasketService = $self->_createBasketService();
    return unless $BasketService->importBasket( $Basket );

    my $Order;
    eval {
        if( $BasketService->wasTriggeredWithShortcutButton() ){
            $self->_shortcutButtonCheckout( $BasketService, $Servlet );
        }else{
            #This step does Basket2Order, since the order is now a new object we have to return this object.
            $Order = $self->_classicOrMultistepCheckout( $BasketService, $Servlet );
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        #rethrow error, since it will be used for automatic form handling
        if( $Error->code() eq 'INVALID_FORM' ){
            Error($Error);
        }
        $self->_LogError( $Error );
        $self->_addErrorToServlet( $Servlet, $Error );
    }

    return $Order;
}
#========================================================================================
# function     _IsAllowedRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       _IsAllowedRequest( $Servlet );
#----------------------------------------------------------------------------------------
# description  checks if the request is allowed
#----------------------------------------------------------------------------------------
# input        $Servlet | epages servlet  | DE_EPAGES::Presentation::Servlet::Permission
# return       $IsAllowedRequest | is allowed request | boolean
#========================================================================================
sub _IsAllowedRequest {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $IsAllowedRequest = 0;
    my $Basket = $Servlet->object();
    my $BasketUser = $Basket->get( 'User' );
    my $ServletUser = $Servlet->user();
    if( defined $ServletUser && $ServletUser->id() eq $BasketUser->id() ){
        $IsAllowedRequest = 1;


    }

    return $IsAllowedRequest;
}
#========================================================================================
# function     _shortcutButtonCheckout
# state        private
#----------------------------------------------------------------------------------------
# syntax       _shortcutButtonCheckout( $BasketService, $Servlet );
#----------------------------------------------------------------------------------------
# description  Processes single click button checkout for paypal-express-checkout (shortcut)
#----------------------------------------------------------------------------------------
# input        $BasketService | extracts relevant data from basket | DE_EPAGES::PayPalExpress::API::Webservice::BasketService
# input        $Servlet | epages servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub _shortcutButtonCheckout {
    my $self = shift;
    my ( $BasketService, $Servlet ) = @_;

    my $GetExpressCheckoutDetails = $self->_createWebserviceGetExpressCheckoutDetails();
    my $WasRequestSuccessful = $GetExpressCheckoutDetails->doRequestForShortcutButtonCheckout( $BasketService, $Servlet );
    if( $WasRequestSuccessful ){
        $self->_redirectToOrderConfirmationPage( $Servlet, $BasketService );
    }else{
        $self->_redirectPaypalErrorToErrorPage( $Servlet, $GetExpressCheckoutDetails );
    }

    return;
}
#========================================================================================
# function     _redirectToOrderConfirmationPage
# state        private
#----------------------------------------------------------------------------------------
# syntax       _redirectToOrderConfirmationPage( $Servlet, $BasketService );
#----------------------------------------------------------------------------------------
# description  set the viewaction for the redirect for the OrderConfirmationPage
#               for Classic, mobile or multistep checkout
#----------------------------------------------------------------------------------------
# input        $Servlet | epages servlet | DE_EPAGES::Presentation::Servlet::Permission
# input        $BasketService | extracts relevant data from basket | DE_EPAGES::PayPalExpress::API::Webservice::BasketService
#========================================================================================
sub _redirectToOrderConfirmationPage {
    my $self = shift;
    my ( $Servlet, $BasketService ) = @_;

    my $IsMultistepCheckout = $BasketService->isMultistepCheckout();
    my $IsMobilDevice = $Servlet->params('IsMobile');

    my $ViewAction;
    if ($IsMultistepCheckout and not $IsMobilDevice) {
        LogPayment('PayPalExpress','_redirectToOrderConfirmationPage -------------ViewMultiCheckoutConfirmation----------------',{'ErrorCode' => 1});

        $ViewAction = 'ViewMultiCheckoutConfirmation'; #Multistep checkout
    } else {
        LogPayment('PayPalExpress','_redirectToOrderConfirmationPage -------------ViewOffer----------------',{'ErrorCode' => 1});

        $ViewAction = 'ViewOffer'; #Classic checkout and mobile checkout
    }

    $Servlet->vars('ViewAction', $ViewAction);

    return;
}
#========================================================================================
# function     _classicOrMultistepCheckout
# state        private
#----------------------------------------------------------------------------------------
# syntax       _classicOrMultistepCheckout( $BasketService, $Servlet );
#----------------------------------------------------------------------------------------
# description  Does the payment-process step for classic or multistep checkout
#----------------------------------------------------------------------------------------
# input        $BasketService | extracts relevant data from basket | DE_EPAGES::PayPalExpress::API::Webservice::BasketService
# input        $Servlet | epages servlet | DE_EPAGES::Presentation::Servlet::Permission
# return       $Order | order | DE_EPAGES::Order::API::Object::CustomerOrder
#========================================================================================
sub _classicOrMultistepCheckout {
    my $self = shift;
    my ( $BasketService, $Servlet ) = @_;

    my $GetExpressCheckoutDetails = $self->_createWebserviceGetExpressCheckoutDetails();
    my $WasRequestSuccessful = $GetExpressCheckoutDetails->doRequestForClassicOrMultistepCheckout( $BasketService, $Servlet );
    my $Order;
    if( $WasRequestSuccessful ){
        $Order = $self->_doBasket2Order( $Servlet, $BasketService->getBasket() );
    } else {
        $self->_redirectPaypalErrorToErrorPage( $Servlet, $GetExpressCheckoutDetails );
    }

    return $Order;
}
#========================================================================================
# function     _createBasketService
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createBasketService( );
#----------------------------------------------------------------------------------------
# description  wrapper for BasketService 
#----------------------------------------------------------------------------------------
# return       DE_EPAGES::PayPalExpress::API::Webservice::BasketService
#========================================================================================
sub _createBasketService {
    my $self = shift;
    return DE_EPAGES::PayPalExpress::API::Webservice::BasketService->new();
}
#========================================================================================
# function     _createWebserviceGetExpressCheckoutDetails
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createWebserviceGetExpressCheckoutDetails( );
#----------------------------------------------------------------------------------------
# description  wrapper for GetExpressCheckoutDetails
#----------------------------------------------------------------------------------------
# return       DE_EPAGES::PayPalExpress::API::Webservice::GetExpressCheckoutDetails
#========================================================================================
sub _createWebserviceGetExpressCheckoutDetails {
    my $self = shift;
    return DE_EPAGES::PayPalExpress::API::Webservice::GetExpressCheckoutDetails->new();
}

#========================================================================================
# function     _redirectPaypalErrorToErrorPage
# state        private
#----------------------------------------------------------------------------------------
# syntax       _redirectPaypalErrorToErrorPage( $Servlet, $GetExpressCheckoutDetails );
#----------------------------------------------------------------------------------------
# description  do the redirect for errormessages for webservice errors
#----------------------------------------------------------------------------------------
# input        $Servlet | epages servlet | DE_EPAGES::Presentation::Servlet::Permission
# input        $GetExpressCheckoutDetails | webservice | DE_EPAGES::PayPalExpress::API::Webservice::GetExpressCheckoutDetails
#========================================================================================
sub _redirectPaypalErrorToErrorPage {
    my $self = shift;
    my ( $Servlet, $GetExpressCheckoutDetails ) = @_;

    LogPayment('PayPalExpress', 'Basket _redirectPaypalErrorToErrorPage',{
        'ErrorCode' => $GetExpressCheckoutDetails->getErrorCode()
    });

    $Servlet->vars('ViewAction', 'ViewPayPalExpressFailed' );
    $Servlet->vars('ErrorCode', 'PayPalExpressInvalid' );
    $Servlet->vars('PPEErrorCode', $GetExpressCheckoutDetails->getErrorCode() );
    $Servlet->vars('PayPalMessage', $GetExpressCheckoutDetails->getErrorMessage() );

    return;
}
#========================================================================================
# function     _addErrorToServlet
# state        private
#----------------------------------------------------------------------------------------
# syntax       _addErrorToServlet( $Servlet, $Error );
#----------------------------------------------------------------------------------------
# description  add the Error to the servlet
#----------------------------------------------------------------------------------------
# input        $Servlet | epages servlet | DE_EPAGES::Presentation::Servlet::Permission
# input        $Error | epages error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub _addErrorToServlet {
    my $self = shift;
    my ($Servlet, $Error) = @_;

    LogPayment('PayPalExpress', 'Basket _addErrorToServlet set ViewPayPalExpressFailed',{
        'ErrorGUID' => $Error->guid()
    });

    $Servlet->vars( 'ViewAction', 'ViewPayPalExpressFailed' );
    $Servlet->vars( 'ErrorCode', 'GetExpressCheckoutDetailsError' );
    $Servlet->vars( 'ErrorGUID', $Error->guid() );
    $Servlet->vars( 'ErrorMessage', $Error->shortMessage() );

    return;
}
#==============================================================================
sub PayPalExpressCancel {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object();
    return unless (defined $Servlet->user() and $Servlet->user()->id() == $Basket->get('User')->id());

    ResetBasket($Basket);

    my $Shop = $Basket->get('Shop');
    if ($Shop->get('IsUnityRegistered')) {
        $Servlet->redirect($Basket->get('WebUrlUnitySSL'));
    }

    return;
}
#==============================================================================
sub PayPalExpressCallback {
    my $self = shift;
    my ($Servlet) = @_;
    my $Basket = $Servlet->object;

    return unless $Basket->class->alias eq 'Basket';

    my $hFormValues = $Servlet->params();
    LogPayment('PayPalExpress', 'PayPalExpressCallback input values', $hFormValues);

    my ($PaymentLineItem, $hShippingValues, $Currency) = TestPayPalExpressParametersCallback($Basket, $hFormValues);
    my $Container = $Basket->container;
    my $PaymentMethod = $PaymentLineItem->get('PaymentMethod');

    #--- process and update address data
    my %AddressData = (
        'Street'  => $hShippingValues->{'SHIPTOSTREET'},
        'Street2' => $hShippingValues->{'SHIPTOSTREET2'},
        'Zipcode' => $hShippingValues->{'SHIPTOZIP'},
        'City'    => $hShippingValues->{'SHIPTOCITY'},
        'State'   => $hShippingValues->{'SHIPTOSTATE'},
    );

    my $ShippingCountryCode = $hShippingValues->{'SHIPTOCOUNTRY'};
    if (defined $ShippingCountryCode and ExistsCountryByCode2(uc($ShippingCountryCode))) {
        $AddressData{'CountryID'} = GetPKeyCountryByCode2(uc($ShippingCountryCode));
    }

    my $BillingAddress = $Basket->get('BillingAddress');
    if (defined $BillingAddress) {
        $BillingAddress->set(\%AddressData);
    }
    else {
        $Basket->child('BillingAddress')->delete if ($Basket->existsChild('BillingAddress'));
        $AddressData{'Alias'} = 'BillingAddress';
        my $NewShippingAddress = $Basket->insertChild('Address', \%AddressData);
        $Basket->set({'BillingAddress' => $NewShippingAddress});
    }

    my $ShippingAddress = $Basket->get('ShippingAddress');
    if (defined $ShippingAddress) {
        $ShippingAddress->set(\%AddressData);
    }
    else {
        $Basket->child('ShippingAddress')->delete if ($Basket->existsChild('ShippingAddress'));
        $AddressData{'Alias'} = 'ShippingAddress';
        my $NewShippingAddress = $Basket->insertChild('Address', \%AddressData);
        $Basket->set({'ShippingAddress' => $NewShippingAddress});
    }

    $Basket->set({'IsAddressOK' => 1});

    #--- update container, tax model, shipping method
    CheckSaveShipping($Container);

    my $ShippingLineItem = $Container->get('Shipping');
    my $ShippingMethod = defined $ShippingLineItem ? $ShippingLineItem->get('ShippingMethod') : undef;

    #--- delete shipping if current shipping method isn't possible with PPE
    if (defined $ShippingMethod and not $PaymentMethod->id ~~ [ map { $_->id } @{$ShippingMethod->get('PaymentMethods')} ]) {
        $ShippingLineItem->delete;
        $ShippingLineItem = undef;
        LogPayment('PayPalExpress', 'ShippingLineItem was deleted', 1);
    }

    #--- find a shipping method possible with PPE if necessary
    unless (defined $ShippingLineItem) {
        my $aShippingMethods = $Container->get('ShippingMethods');
        my @PossibleMethods = ();
        LogPayment('PayPalExpress', 'ShippingMethods', $aShippingMethods);

        foreach my $ContainerShippingMethod (@$aShippingMethods) {
            if ($PaymentMethod->id ~~ [ map { $_->id } @{$ContainerShippingMethod->get('PaymentMethods')} ]) {
                push @PossibleMethods, $ContainerShippingMethod;
            }
        }
        LogPayment('PayPalExpress', 'PossibleMethods', \@PossibleMethods);

        my $NewShippingMethod;

        if (scalar @PossibleMethods > 0) {
            my @Defaults = grep { $_->get('IsDefault') } @PossibleMethods;
            $NewShippingMethod = scalar @Defaults ? $Defaults[0] : $PossibleMethods[0];
        }

        if (defined $NewShippingMethod) {
            $NewShippingMethod->addToContainer($Container);
        }
    }

    #--- Payment may be changed due to an address change
    my $NewPaymentMethod = defined $Container->get('Payment') ? $Container->get('Payment')->get('PaymentMethod') : undef;

    #--- check PaymentMethod
    unless (defined $NewPaymentMethod and $NewPaymentMethod->id == $PaymentMethod->id) {
        my $ExistsPayPalExpress = 0;
        my $aPaymentMethods = $Container->getSite->get('PaymentMethods');
        foreach my $ShopPaymentMethod (@$aPaymentMethods) {
            if ($ShopPaymentMethod->instanceOf('PaymentMethodPayPalExpress')) {
                $NewPaymentMethod = $ShopPaymentMethod;
                $ExistsPayPalExpress = 1;
                last;
            }
        }

        if ($ExistsPayPalExpress) {
            #--- change payment method
            my $hOldPliValues = $PaymentLineItem->get(['Token', 'Field1', 'IsExpressCheckout']);
            $PaymentLineItem->delete;
            $NewPaymentMethod->addToContainer($Container, {});

            #--- clone important attributes to new PayPalExpress line item
            my $NewPaymentLineItem = $Container->get('Payment');
            $NewPaymentLineItem->set($hOldPliValues);

            LogPayment('PayPalExpress', 'Original pli', $PaymentLineItem);
            LogPayment('PayPalExpress', 'Changed pli', $NewPaymentLineItem);
        }
        else {
            LogPayment('PayPalExpress', 'No PayPalExpress method available', $NewPaymentMethod);
        }
    }

    UpdateTaxModel($Basket->container);
    $Basket->updateTaxArea;
    $Basket->recalculate;


    #--- response with updated values
    my %InstantUpdateParams = (
        'METHOD'               => 'CallbackResponse',
        'CALLBACKVERSION'      => API_VERSION,
        'OFFERINSURANCEOPTION' => 'false',
        'CURRENCYCODE'         => $Currency,
        %{GetInstantUpdateParams($Container)},
    );

    my @Content = ();

    foreach my $Param (keys %InstantUpdateParams) {
        push @Content, join('=', $Param, uri_escape_utf8($InstantUpdateParams{$Param}));
    }

    my $Response = join('&', @Content);
    LogPayment('PayPalExpress', 'CallbackResponse', $Response);

    $Servlet->vars('ExitAfterEvent', 1);
    $Servlet->vars('OutputEncoding', 'utf-8');
    $Servlet->vars('ContentType', 'application/x-www-form-urlencoded');
    $Servlet->writeContentOutput($Response);

    return;
}
#==============================================================================
sub TestPayPalExpressParametersCallback {
    my ($Basket, $hFormValues) = @_;

    return unless $Basket->class->alias eq 'Basket';

    my $Shop = $Basket->getSite();

    my $BasketGUID = $hFormValues->{'BasketGUID'};
    Error('BadRedirect_NoBasketGUID', {
        'Name'       => 'BasketGUID',
        'Reason'     => 'BadRedirect_NoBasketGUID',
    }) unless (defined $BasketGUID and $Basket->get('GUID') eq $BasketGUID);
    my $PaymentLineItem = $Basket->container->get('Payment');

    Error('BadRedirect_ShopIDMisMatch', {
        'Name'       => 'ShopID',
        'Reason'     => 'BadRedirect_ShopIDMisMatch'
    }) if ($PaymentLineItem->getSite()->id != $Shop->id);

    # extract and return shipping form values
    my %ShippingValues = ();
    foreach my $Param (qw(SHIPTOSTREET SHIPTOCITY SHIPTOSTATE SHIPTOCOUNTRY SHIPTOZIP SHIPTOSTREET2)) {
        $ShippingValues{$Param} = $hFormValues->{$Param};
    }

    # capture currency code
    my $Currency = $hFormValues->{'CURRENCYCODE'};

    return ($PaymentLineItem, \%ShippingValues, $Currency);
}

#========================================================================================
# function     LogError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_LogError( $Error );
#----------------------------------------------------------------------------------------
# description  Writes an error to the log file.
#----------------------------------------------------------------------------------------
# input        $Error | an error object | DE_EPAGES::Core::API::Error
# return       1 | always 1 | boolean
#========================================================================================
sub _LogError {
    my $self = shift;
    my ( $Error ) = @_;
    return LogError( $Error );
}
#========================================================================================
# function     _doBasket2Order
# state        private
#----------------------------------------------------------------------------------------
# syntax       _doBasket2Order( $Servlet, $Basket );
#----------------------------------------------------------------------------------------
# description  wrapper for core method doBasket2Order
#----------------------------------------------------------------------------------------
# input        $Servlet | epages servlet | DE_EPAGES::Presentation::Servlet::Permission
# input        $Basket | epages basket | DE_EPAGES::Order::API::Object::Basket
# return       $Order | order | DE_EPAGES::Order::API::Object::CustomerOrder
#========================================================================================
sub _doBasket2Order {
    my $self = shift;
    my ( $Servlet, $Basket ) = @_;
    my $Order = DE_EPAGES::Order::UI::Basket->doBasket2Order( $Servlet, $Basket );
    return $Order;
}

1;

