#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShopping view setup
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use strict;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef String );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );

use Test::More;
use Test::Exception;

use constant MOCKED_SERVLET_PARAMS_WITH_GUID => {'PaymentGUID' => 'lolcat'};
use constant MOCKED_FORM_VALUES => {'TransStatus' => 'yes', 'TransID' => '#1'};

sub test {
    my $self = shift;

    $self->test_PayPalExpressIPN();
    $self->test_PayPalExpressIPN_Dupe();
    $self->test_PayPalExpressIPN_UnknownGUID();
    $self->test_PayPalExpressIPN_UnknownGUID_TestShop();
    $self->test_PayPalExpressIPN_NoGUID();

    done_testing();
}

sub test_PayPalExpressIPN {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedServletParams = MOCKED_SERVLET_PARAMS_WITH_GUID;
    my $MockedFormValues = MOCKED_FORM_VALUES;
    my $MockedShop = $self->_createShop();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedServletParams);
    my $MockedPaymentLineItem = $self->_createLineItemPaymentPayPalExpress();
    $MockedFormValues->{'PaymentLineItem'} = $MockedPaymentLineItem;

    my $SUT = $self->_createSUT({
        'PaymentLineItem' => $MockedPaymentLineItem,
        'FormValues'      => $MockedFormValues
    });

    lives_ok (
        sub { $SUT->PayPalExpressIPN($MockedServlet); },
        $SubTestName . ' - Verify that PayPalExpressIPN does not throw an error'
    );

    ok (WasCalled ($SUT, 'ExistsObjectByGUID'), $SubTestName . ' - Verify that ExistsObjectByGUID was called');
    ok (WasCalled ($SUT, 'LoadObjectByGUID'), $SubTestName . ' - Verify that LoadObjectByGUID was called');

    my $verifyWasCalled = WasCalled ($SUT, '_verifyPayPalExpressIPN');
    ok ($verifyWasCalled, $SubTestName . ' - Verify that _verifyPayPalExpressIPN was called');
    return unless ($verifyWasCalled == 1);

    my $IsInTestMode = GetParametersFromMockifyCall($SUT, '_verifyPayPalExpressIPN')->[1];
    is ($IsInTestMode, 0, $SubTestName . ' - Verify sandbox URL is NOT being used');
    ok (WasCalled ($SUT, 'TestPayPalExpressParametersIPN'), $SubTestName . ' - Verify that TestPayPalExpressParametersIPN was called');

    my $Object = $MockedPaymentLineItem->container->parent;
    ok (WasCalled ($Object, 'instanceOf'), $SubTestName . ' - Verify that instanceOf was called on $Object');
    ok (WasCalled ($SUT, '_SetTransStatusOnOrder'), $SubTestName . ' - Verify that _SetTransStatusOnOrder was called');
    is(
        $MockedPaymentLineItem->get('LastIPNTransID'), $MockedFormValues->{'TransID'},
        $SubTestName . ' - Verify LastIPNTransID was set correctly'
    );
    is(
        $MockedPaymentLineItem->get('LastIPNTransStatus'), $MockedFormValues->{'TransStatus'},
        $SubTestName . ' - Verify LastIPNTransStatus was set correctly'
    );
}

sub test_PayPalExpressIPN_Dupe {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedServletParams = MOCKED_SERVLET_PARAMS_WITH_GUID;
    my $MockedFormValues = MOCKED_FORM_VALUES;
    my $MockedShop = $self->_createShop();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedServletParams);
    my $MockedPaymentLineItem = $self->_createLineItemPaymentPayPalExpress();
    $MockedFormValues->{'PaymentLineItem'} = $MockedPaymentLineItem;

    $MockedPaymentLineItem->set({'LastIPNTransID' => $MockedFormValues->{'TransID'}});
    $MockedPaymentLineItem->set({'LastIPNTransStatus' => $MockedFormValues->{'TransStatus'}});

    my $SUT = $self->_createSUT({
        'PaymentLineItem' => $MockedPaymentLineItem,
        'FormValues'      => $MockedFormValues
    });

    lives_ok (
        sub { $SUT->PayPalExpressIPN($MockedServlet); },
        $SubTestName . ' - Verify that PayPalExpressIPN does not throw an error'
    );

    ok (WasCalled ($SUT, 'ExistsObjectByGUID'), $SubTestName . ' - Verify that ExistsObjectByGUID was called');
    ok (WasCalled ($SUT, 'LoadObjectByGUID'), $SubTestName . ' - Verify that LoadObjectByGUID was called');

    my $verifyWasCalled = WasCalled ($SUT, '_verifyPayPalExpressIPN');
    ok ($verifyWasCalled, $SubTestName . ' - Verify that _verifyPayPalExpressIPN was called');
    return unless ($verifyWasCalled == 1);

    my $IsInTestMode = GetParametersFromMockifyCall($SUT, '_verifyPayPalExpressIPN')->[1];
    is ($IsInTestMode, 0, $SubTestName . ' - Verify sandbox URL is NOT being used');
    ok (WasCalled ($SUT, 'TestPayPalExpressParametersIPN'), $SubTestName . ' - Verify that TestPayPalExpressParametersIPN was called');

    my $Object = $MockedPaymentLineItem->container->parent;
    ok (! WasCalled ($Object, 'instanceOf'), $SubTestName . ' - Verify that instanceOf was NOT called on $Object');
    ok (! WasCalled ($SUT, '_SetTransStatusOnOrder'), $SubTestName . ' - Verify that _SetTransStatusOnOrder was NOT called');
}

sub test_PayPalExpressIPN_UnknownGUID {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedServletParams = MOCKED_SERVLET_PARAMS_WITH_GUID;
    my $MockedFormValues = MOCKED_FORM_VALUES;
    my $MockedShop = $self->_createShop();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedServletParams);

    my $SUT = $self->_createSUT({
        'FormValues'      => $MockedFormValues
    });

    lives_ok (
        sub { $SUT->PayPalExpressIPN($MockedServlet); },
        $SubTestName . ' - Verify that PayPalExpressIPN does not throw an error'
    );

    ok (WasCalled ($SUT, 'ExistsObjectByGUID'), $SubTestName . ' - Verify that ExistsObjectByGUID was called');
    ok (! WasCalled ($SUT, 'LoadObjectByGUID'), $SubTestName . ' - Verify that LoadObjectByGUID was NOT called');

    my $verifyWasCalled = WasCalled ($SUT, '_verifyPayPalExpressIPN');
    ok ($verifyWasCalled, $SubTestName . ' - Verify that _verifyPayPalExpressIPN was called');
    return unless ($verifyWasCalled == 1);

    my $IsInTestMode = GetParametersFromMockifyCall($SUT, '_verifyPayPalExpressIPN')->[1];
    is ($IsInTestMode, 0, $SubTestName . ' - Verify sandbox URL is NOT being used');
    ok (! WasCalled ($SUT, 'TestPayPalExpressParametersIPN'), $SubTestName . ' - Verify that TestPayPalExpressParametersIPN was NOT called');
}

sub test_PayPalExpressIPN_UnknownGUID_TestShop {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedServletParams = MOCKED_SERVLET_PARAMS_WITH_GUID;
    my $MockedFormValues = MOCKED_FORM_VALUES;
    my $MockedShop = $self->_createShop({ 'IsInternalTestShop' => 1});
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedServletParams);

    my $SUT = $self->_createSUT({
        'FormValues'      => $MockedFormValues
    });

    lives_ok (
        sub { $SUT->PayPalExpressIPN($MockedServlet); },
        $SubTestName . ' - Verify that PayPalExpressIPN does not throw an error'
    );

    ok (WasCalled ($SUT, 'ExistsObjectByGUID'), $SubTestName . ' - Verify that ExistsObjectByGUID was called');
    ok (! WasCalled ($SUT, 'LoadObjectByGUID'), $SubTestName . ' - Verify that LoadObjectByGUID was NOT called');

    my $verifyWasCalled = WasCalled ($SUT, '_verifyPayPalExpressIPN');
    ok ($verifyWasCalled, $SubTestName . ' - Verify that _verifyPayPalExpressIPN was called');
    return unless ($verifyWasCalled == 1);

    my $IsInTestMode = GetParametersFromMockifyCall($SUT, '_verifyPayPalExpressIPN')->[1];
    is ($IsInTestMode, 1, $SubTestName . ' - Verify sandbox URL is being used');
    ok (! WasCalled ($SUT, 'TestPayPalExpressParametersIPN'), $SubTestName . ' - Verify that TestPayPalExpressParametersIPN was NOT called');
}

sub test_PayPalExpressIPN_NoGUID {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedServletParams = {  };
    my $MockedFormValues = MOCKED_FORM_VALUES;
    my $MockedShop = $self->_createShop();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedServletParams);

    my $SUT = $self->_createSUT({
        'FormValues'      => $MockedFormValues
    });

    lives_ok (
        sub { $SUT->PayPalExpressIPN($MockedServlet); },
        $SubTestName . ' - Verify that PayPalExpressIPN does not throw an error'
    );

    ok (! WasCalled ($SUT, 'ExistsObjectByGUID'), $SubTestName . ' - Verify that ExistsObjectByGUID was NOT called');
    ok (! WasCalled ($SUT, 'LoadObjectByGUID'), $SubTestName . ' - Verify that LoadObjectByGUID was NOT called');

    my $verifyWasCalled = WasCalled ($SUT, '_verifyPayPalExpressIPN');
    ok ($verifyWasCalled, $SubTestName . ' - Verify that _verifyPayPalExpressIPN was called');
    return unless ($verifyWasCalled == 1);

    my $IsInTestMode = GetParametersFromMockifyCall($SUT, '_verifyPayPalExpressIPN')->[1];
    is ($IsInTestMode, 0, $SubTestName . ' - Verify sandbox URL is NOT being used');
    ok (! WasCalled ($SUT, 'TestPayPalExpressParametersIPN'), $SubTestName . ' - Verify that TestPayPalExpressParametersIPN was NOT called');
}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($hParams) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::PayPalExpress::UI::Shop', []);

    # static mocks
    $MockifySut->mockImported('DE_EPAGES::ExternalPayment::API::Log', 'LogPayment')
        ->when(String(), String(), HashRef())->thenReturnUndef();
    $MockifySut->mockStatic('DE_EPAGES::Order::UI::Basket::doBasket2Order')
        ->whenAny()
        ->thenReturnUndef();

    if (defined($hParams->{'PaymentLineItem'})) {
        $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadObjectByGUID')
            ->when(String())->thenReturn($hParams->{'PaymentLineItem'});
        $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'ExistsObjectByGUID')
            ->when(String())->thenReturn(1);
    } else {
        $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadObjectByGUID')
            ->when(String())->thenThrowError("oh no");
        $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'ExistsObjectByGUID')
            ->when(String())->thenReturn(0);
    }
    if (defined($hParams->{'FormValues'})) {
        $MockifySut->mock('TestPayPalExpressParametersIPN')
            ->whenAny()
            ->thenReturn($hParams->{'FormValues'});
    } else {
        $MockifySut->mock('TestPayPalExpressParametersIPN')
            ->whenAny()
            ->thenReturnUndef();
    }

    # mock privates
    $MockifySut->mock('_verifyPayPalExpressIPN')
        ->whenAny()
        ->thenReturn(1);
    $MockifySut->mock('_SetTransStatusOnOrder')
        ->whenAny()
        ->thenReturnUndef();

    return $MockifySut->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($Shop, $hServletParams) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('params', $hServletParams);
    $MockifyServlet->addMockWithReturnValue('request', {});

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'ID'                 => undef,
        'IsInternalTestShop' => 0,
        %{ $hParams // {} },
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    my $Shop  = $MockifyShop->getMockObject();
    $Shop->set({'Site' => $Shop});

    return $Shop;
}

sub _createLineItemPaymentPayPalExpress {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'PaymentMethod'      => $self->_createPaymentMethod($hParams),
        #'instanceOf' => LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS,
        'TransStatus'        => 'yes',
        'LastIPNTransStatus' => 'no',
        'LastIPNTransID'     => undef,
        #'MollieOrderId' => MOLLIE_ID,
        'TransTime'          => undef,
        'TransID'            => 0,
        'LineItemContainer'  => $self->_createLineItemContainer(),
        'IsTestTransaction'  => 1,
        'ID'                 => 21348,
    };
    my $MockifyLineItemPayment = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress', $hAttributes);
    return $MockifyLineItemPayment->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );

    $MockifyContainer->addMockWithReturnValue('parent', $self->_createCustomerOrder());

    return $MockifyContainer->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentMethod {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'ID'           => 123,
        'instanceOf'   => 'PaymentMethodPayPalExpress',
        'IsInTestMode' => 0,
        'DoPaymentOnBasket'  => 0,
        %{ $hParams // {} },
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::PayPalExpress::API::Object::PaymentMethodPayPalExpress', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

sub _createCustomerOrder {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyCustomerOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );

    $MockifyCustomerOrder->addMockWithReturnValueAndParameterCheck('instanceOf', 1, ['string']);

    return $MockifyCustomerOrder->getMockObject();
}

__PACKAGE__->runScript();
