#========================================================================================
# package      DE_EPAGES::PayPalPPCP::API::TLE::PayPalPPCPHandlerHandler
# state        public
#----------------------------------------------------------------------------------------
# description Implements a TLE Function which returns preformatted PayPalPPCP widget HTML code
#========================================================================================
package DE_EPAGES::PayPalPPCP::API::TLE::PayPalPPCPHandler;

use strict;
use JSON;
use MIME::Base64;
use DE_EPAGES::Presentation::API::DereferrerToken qw (GenerateDereferrerToken);
use DE_EPAGES::Payment::API::Table::ShippingPaymentMap qw (
    ExistsShippingPaymentMap
);

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::PayPalPPCP::API::TLE::PayPalPPCPHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift;}

#========================================================================================
# function     tle
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->tle( $Processor, $TLEName);
# example      my $Value = $Handler->tle( $Processor, 'Name');
#----------------------------------------------------------------------------------------
# description  This function is called by tle processor to get the value of a tle
#               variable.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | return value | integer
#========================================================================================
sub tle {
    my $self = shift;
    my ($Processor, $TLEName) = @_;

    my $Object = $Processor->tle('Object');
    my $Shop = $Object->getSite();

    if( $TLEName eq 'ExtractedPayPalPPCPParameters' ) {
        my $ClassAlias = 'PaymentMethodPayPalPPCP';
        my ($PaymentMethod) = grep {$_->class->alias eq $ClassAlias && $_->get('IsActivated') } @{$Shop->getSite()->get('PaymentMethods')};

        return undef if ! $PaymentMethod;
        return undef if ! $PaymentMethod->hasAppCredentials();

        my $CreateOrderUrl   = $self->_getOrderActionURL($Shop, 'Create');
        my $ApproveOrderUrl = $self->_getOrderActionURL($Shop, 'Approve');

        my $hPaymentMethodInfo = $PaymentMethod->get(['ClientID', 'MerchantInfoJSON', 'EnablePayLater', 'EnablePayLaterBanners']);
        my $PartnerBNCode = $PaymentMethod->getBNCode();

        #-- Minimum/Maximum amounts for the payment method. The basket has to be between these amounts for the buttons to show.
        my $aShopCurrencies = $Shop->get('Currencies');
        my $hMinMaxAllCurrencies = {};
        foreach my $hCurrencyInfo ( @$aShopCurrencies ) {
            next if ! $hCurrencyInfo->{'IsActivated'};
            next if ! $hCurrencyInfo->{'IsUsed'};

            $hMinMaxAllCurrencies->{$hCurrencyInfo->{'CurrencyID'}} = {
                MinPriceValue => $PaymentMethod->getMinPriceValue($hCurrencyInfo->{'CurrencyID'}),
                MaxPriceValue => $PaymentMethod->getMaxPriceValue($hCurrencyInfo->{'CurrencyID'}),
            };
        }

        my $ShippingMethods = $Shop->get('ShippingMethods');
        my $ahLanguages = $Shop->get('Languages');
        my $hIncompatibleShippingMethodNames = {};

        #-- Shipping/Payment method dependencies. If the cart has a shipping method that is NOT available with PPCP, we want to know about it
        foreach my $ShippingMethod (@$ShippingMethods) {
            #-- This means, this shipping method is NOT compatible with our PPCP payment method
            if ( ! ExistsShippingPaymentMap($ShippingMethod->id, $PaymentMethod->id) ) {
                foreach my $hInfo ( @$ahLanguages ) {
                    my $NameOrAlias = $ShippingMethod->get('NameOrAlias', $hInfo->{'LanguageID'});
                    $hIncompatibleShippingMethodNames->{ encode_base64($NameOrAlias, '') } = 1;
                }
            }
        }

        return {
            'ClientID'          => $hPaymentMethodInfo->{ 'ClientID'},
            'MerchantID'        => $hPaymentMethodInfo->{'MerchantInfoJSON'}->{'merchant_id'},
            'CurrencyID'        => $Shop->get('CurrencyID'),
            'PartnerBNCode'     => $PartnerBNCode,
            'EnablePayLater'    => $hPaymentMethodInfo->{ 'EnablePayLater'},
            'EnablePayLaterBanners'    => $hPaymentMethodInfo->{ 'EnablePayLaterBanners'},
            'ApprovePPCPToken'  => $self->_generatePickupToken($ApproveOrderUrl),
            'ApproveOrderUrl'   => $ApproveOrderUrl,
            'CreateOrderUrl'    => $CreateOrderUrl,
            'MinMaxAllCurrencies' => to_json($hMinMaxAllCurrencies),
            'InCompatibleShippingMethods' => to_json($hIncompatibleShippingMethodNames),
        };
    }

    #-- https://developer.paypal.com/limited-release/fraudnet/integrate/
    if ( $TLEName eq 'PayPalPPCPFraudNetFParameter' ) {
        if ( $Object->instanceOf('Basket') ) {
            my $PaymentLine = $Object->container->get('Payment');
            if ( $PaymentLine->instanceOf('LineItemPaymentPayPalPPCP') ) {
                my $PaymentMethod = $PaymentLine->get('PaymentMethod');
                my $FParameter = $PaymentMethod->getFraudNetFParameter($PaymentLine);
                return $FParameter;
            }
        }

        return '';
    }

    #-- https://developer.paypal.com/sdk/js/configuration/#integration-date
    if ( $TLEName eq 'PayPalPPCPIntegrationDate' ) {
        return '2022-03-24';
    }

    return undef;
}

#========================================================================================
# function     existsTLE
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->existsTLE( $Processor, $TLEName);
# example      my $Value = $Handler->existsTLE( $Processor, 'Products_Count');
#----------------------------------------------------------------------------------------
# description  This function is called by the tle processor to test if a TLE variable
#               is available.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | yes or no | bool
#========================================================================================
sub existsTLE {
    my $self = shift;
    my ($Processor, $TLEName) = @_;

    return (defined $self->tle($Processor, $TLEName)) ? 1 : 0;
}

#========================================================================================
# function     PPCP_CanPUI
# state        public
#----------------------------------------------------------------------------------------
# syntax       #PPCP_CanPUI(#PaymentMethod)
# example      #PPCP_CanPUI(#PaymentMethod)
#----------------------------------------------------------------------------------------
# description  Check if this payment method supports PUI
#----------------------------------------------------------------------------------------
# input        #PaymentMethod | A PPCP Payment Method | object
# return       1/0 | Whether PUI is supported | boolean
#========================================================================================
sub PPCP_CanPUI { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($PaymentMethod, $Container) = @$aParams;

    return 0 unless $PaymentMethod && $PaymentMethod->instanceOf('PaymentMethodPayPalPPCP');

    return $PaymentMethod->canUsePUI($Container);
}

#========================================================================================
# function     GetBNCode
# state        public
#----------------------------------------------------------------------------------------
# syntax       #GetBNCode(#PaymentMethod)
# example      #GetBNCode(#PaymentMethod)
#----------------------------------------------------------------------------------------
# description  Get the BN Code for this ppcp payment method
#----------------------------------------------------------------------------------------
# input        #PaymentMethod | A PPCP Payment Method | object
# return       1/0 | Whether PUI is supported | boolean
#========================================================================================
sub GetBNCode { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($PaymentLine) = @$aParams;

    my $PaymentMethod = $PaymentLine->get('PaymentMethod');
    return '' unless $PaymentMethod && $PaymentMethod->instanceOf('PaymentMethodPayPalPPCP');

    return $PaymentMethod->getBNCode($PaymentLine);
}

sub register {
    my $self = shift;
    my ($Processor) = @_;

    $Processor->registerHandler( 'VariableHandler', $self, 'ExtractedPayPalPPCPParameters');
    $Processor->registerHandler( 'VariableHandler', $self, 'PayPalPPCPFraudNetFParameter');
    $Processor->registerHandler( 'VariableHandler', $self, 'PayPalPPCPIntegrationDate');
    $Processor->registerHandler( 'FunctionHandler', $self, 'PPCP_CanPUI' );
    $Processor->registerHandler( 'FunctionHandler', $self, 'GetBNCode' );

    return;
}

sub _getOrderActionURL {
    my $self = shift;
    my ($Shop, $ActionName) = @_;

    my $Url = $Shop->get('WebServerScriptUrlSSL');

    return "$Url?ViewAction=JSONViewResponse&ChangeAction=JSON" . $ActionName . "PayPalPPCPOrder";
}

sub _generatePickupToken {
    my $self = shift;
    my ( $PickupTokenUrl ) = @_;

    return GenerateDereferrerToken($PickupTokenUrl);
}

1;

