#==============================================================================
# §package      DE_EPAGES::PayPalPPCP::Hooks::OrderProcess
# §state        private
#==============================================================================
package DE_EPAGES::PayPalPPCP::Hooks::OrderProcess;

use DE_EPAGES::ExternalPayment::API::Constants qw ( DEFAULT_PAYMENT_STEP_POSITION );
use DE_EPAGES::PayPalPPCP::API::Constants qw ( $INVOICE_FS );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Order::API::LineItemContainer qw( UpdateTaxModel );
use DE_EPAGES::Presentation::API::Session qw( SetSessionAttribute );

use strict;

#========================================================================================
# §function     OnRegisterCheckoutProcess
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       OnRegisterCheckoutProcess($hParams);
#----------------------------------------------------------------------------------------
# §description  register payment step for checkout process
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    my $BasketOrder = $Servlet->object();
    my $LineItemContainer = $BasketOrder->container();
    return unless defined $LineItemContainer;

    my $LineItemPayment = $LineItemContainer->get('Payment');
    return unless defined($LineItemPayment);

    if ($LineItemPayment->instanceOf('LineItemPaymentPayPalPPCP') && !$LineItemPayment->get('PayViaPUI')) {
        $BasketOrderProcess->addStep('PayPalPPCP', DEFAULT_PAYMENT_STEP_POSITION);
    }

    return;
}

#========================================================================================
# §function     AfterRegisterCheckoutProcess
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       AfterRegisterCheckoutProcess($hParams);
#----------------------------------------------------------------------------------------
# §description  set order step; remove payment step if necessary
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub AfterRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    my $BasketOrder = $Servlet->object();
    my $LineItemContainer = $BasketOrder->container();
    return unless defined $LineItemContainer;

    my $LineItemPayment = $LineItemContainer->get('Payment');
    return unless defined $LineItemPayment;

    if ($LineItemPayment->instanceOf('LineItemPaymentPayPalPPCP') && !$LineItemPayment->get('PayViaPUI')) {
        if ($LineItemPayment->get('PaymentMethod')->get('DoPaymentOnBasket')) {
            $BasketOrderProcess->setOrderStep('PayPalPPCP');
        }
    } else {
        $BasketOrderProcess->deleteStep('PayPalPPCP');
    }

    return;
}

#========================================================================================
# §function     OnUI_ViewMultiCheckoutPayment
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       OnUI_ViewMultiCheckoutPayment($hParams);
#----------------------------------------------------------------------------------------
# §description  Executed when viewing the payment page of the individual steps checkout
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnUI_ViewMultiCheckoutPayment {
    my ($hParams) = @_;
}

#========================================================================================
# §function     OnUI_AfterSaveMultiCheckoutPayment
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       OnUI_AfterSaveMultiCheckoutPayment($hParams);
#----------------------------------------------------------------------------------------
# §description  Executed after clicking 'Next' on the payment page of the individual steps checkout
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnUI_AfterSaveMultiCheckoutPayment {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};

    my $Basket = $Servlet->object;
    return if $Basket->get('OrderGUID');

    my $Container = $Basket->container;
    return if ! defined $Container;

    my $PaymentLine = $Container->get('Payment');
    return if ! defined $PaymentLine;
    return unless $PaymentLine->instanceOf('LineItemPaymentPayPalPPCP');

    my $PaymentMethod = $PaymentLine->get('PaymentMethod');

    my $Form = $Servlet->form;

    my $FundingSource = $Form->value('FundingSource');
    if ( $FundingSource && $FundingSource =~ /\S/ ) {

        #-- Verify again that PUI can be used, just in case
        if ( $FundingSource eq $INVOICE_FS ) {
            $FundingSource = undef if ! $PaymentMethod->canUsePUI($PaymentMethod, $PaymentLine->container);
        }

        $PaymentLine->set({
            'FundingSource' => $FundingSource,
        });
    } else {
        $PaymentLine->set({
            'FundingSource' => undef,
        });
    }

    $PaymentLine->set({
        'PayViaPUI' => $FundingSource eq $INVOICE_FS ? 1 : 0,
    });

    #-- This is initially created by the calling function, but at that point, 'PayViaPUI' has not been saved yet, so our Register/AfterRegister CheckoutProcess hooks will not have run correctly
    #-- Therefore, run it again by (re)creating the BasketOrderProcess object, which triggers off those hooks
    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess',$BasketOrderProcess->tleHash);

    #-- Verify birthday/phone
    if ( $FundingSource eq $INVOICE_FS ) {
        my $AddressClass = LoadClassByAlias('Address');
        my $Shop = $PaymentMethod->getSite();

        my $HasError = 0;
        my $hInfo;
        foreach my $AttrAlias ( qw/Birthday Phone/ ) {
            my $Attribute = $AddressClass->attribute($AttrAlias);

            my $hFieldDef = { Name => $AttrAlias, Form => 'Billing' . 'Address' };
            $hFieldDef->{Mandatory} = 1;
            $hFieldDef->{Type} = 'reg_date' if $AttrAlias eq 'Birthday';
            $hFieldDef->{Type} = 'string' if $AttrAlias eq 'Phone';
            $hFieldDef->{MaxValue} = $Attribute->get('Length') if $AttrAlias eq 'Phone' && $Attribute->get('Length');

            my $OrgValue = $Form->value( $AttrAlias );

            my $Val = $Form->field( $Shop, $OrgValue, $hFieldDef );
            #$PaymentMethod->logPPCP('Val', $Val);

            if ( ! defined $Val ) {
                $HasError = 1;
                #$PaymentMethod->logPPCP('HasError', $HasError);
            } else {
                $hInfo->{$AttrAlias} = $Val;
            }
        }

        #$PaymentMethod->logPPCP('formerrors', $Form->formErrors());

        #-- I think other handlers may have already added their form errors, so in order
        #-- for ours to take precedence, we need to clear all existing form errors??
        if ( $HasError ) {
            $Form->resetFormErrors();
            $Form->executeFormError({
                'ViewAction' => 'ViewMultiCheckoutPayment',
                'Name' => 'MAND_FIELDS',
                'Reason' => 'PUI_INVALID_MAND_FIELDS',
                'Form' => 'SavePUI',
            });
        } else {
            my $BillingAddress = $Basket->get('BillingAddress');
            $BillingAddress->set($hInfo) if $BillingAddress;
        }
    }
}

#========================================================================================
# §function     OnUI_ViewMultiCheckoutConfirmation
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       OnUI_ViewMultiCheckoutConfirmation($hParams);
#----------------------------------------------------------------------------------------
# §description  Executed when viewing the checkout confirmation page of the individual steps checkout
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnUI_ViewMultiCheckoutConfirmation {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};

    my $Basket = $Servlet->object;
    my $Shop = $Basket->getSite();

    #-- Unless we are coming from the express button
    my $Fppcp = $Servlet->form->value('Fppcp');
    return if ! $Fppcp;

    return if $Basket->get('OrderGUID');

    my $Container = $Basket->container;
    return if ! defined $Container;

    my $CurrentGrandTotal = $Container->get('GrandTotal');

    #=================================
    #-- Taken from Order::UI::OrderProcess::SaveMultiCheckoutAddress
    #=================================
    # update TaxModel, if address was changed
    my $TaxModelChanged = UpdateTaxModel($Container);
    if (defined $TaxModelChanged) {
        $Container->recalculate();
        SetSessionAttribute( $Servlet->intSessionID, 'TaxModelChanged', 1) unless $Shop->get('VATreleased');
    }

    my $TaxChanged;
    my $TaxChangedDigital;

    # test if we have a TaxMethod
    if($Shop->get('TaxMethod')){
        $Shop->get('TaxMethod')->updateContainerTaxArea($Container);
    }else {
        # update tax area, if address changed
        $TaxChanged = $Basket->updateTaxArea();
        $TaxChangedDigital = $Basket->updateTaxAreaDigital();
    }

    $Servlet->params('TaxChanged', 1) if $TaxChanged;
    $Servlet->params('TaxChangedDigital', 1) if $TaxChangedDigital;
    #=================================
    #-- Taken from Order::UI::OrderProcess::SaveMultiCheckoutAddress
    #=================================

    #-- When checking out for the first time using the express button, the cart's grand total is dependent on the dt (delivery threshold) country switch, if it is enabled
    #-- and since there is no existing address in the cart, the code above will not register 'TaxChanged'
    #-- Since it is difficult and time consuming to compare with the current value of the dt country switch, just compare the grand totals
    #-- Yeah it's not entirely accurate because it will also trigger this message if they just add/remove products in the meantime, for e.g., but for the most part ...
    my $NewGrandTotal = $Container->get('GrandTotal');
    if ( $CurrentGrandTotal != $NewGrandTotal ) {
        $Servlet->params('TaxChanged', 1);
        $Servlet->params('TaxChangedDigital', 1);
    }
}

1;
