#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request
# state        public
#----------------------------------------------------------------------------------------
# description  class to prepare the request for the CreatePayment call for PayPal plus
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request;
use base DE_EPAGES::PayPalPlus::API::Webservice::Base::Repository;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestFloat TestObject TestString );
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::Request;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::LineItem;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Discount;
use DE_EPAGES::PayPalPlus::API::Constants qw ( PAYPALPLUS_IPN_CALLBACK_PATH );
use DE_EPAGES::Shop::API::Constants qw( TAX_MODEL_NET );
use DE_EPAGES::Shop::API::ShopUriBuilder;
use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {
        'CreatePaymentRequestDTO'=> undef,
    }, $class;
    return $self;
}
#========================================================================================
# function     collectData
# state        public
#----------------------------------------------------------------------------------------
# syntax       collectData( $Basket );
#----------------------------------------------------------------------------------------
# description  Collect relevant information from basket to create the RequestDTO object
#----------------------------------------------------------------------------------------
# input       $Basket | Basket object | DE_EPAGES::Order::API::Object::Basket
# return      boolean flag if RequestDTO was successful created | boolean
#========================================================================================
sub collectData {
    my $self = shift;
    my ( $Basket ) = @_;

    TestObject('Basket', $Basket, 'DE_EPAGES::Order::API::Object::Basket');

    my $LineItemContainer = $Basket->container();
    TestObject('LineItemContainer', $LineItemContainer, 'DE_EPAGES::Order::API::Object::LineItemContainer');

    my $LineItemPaymentPayPalPlus = $LineItemContainer->get('Payment');
    TestObject('LineItemPaymentPayPalPlus', $LineItemPaymentPayPalPlus, 'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus');

    my $PaymentMethodPayPalPlus = $LineItemPaymentPayPalPlus->get('PaymentMethod');
    $self->buildCredentialDTO($PaymentMethodPayPalPlus);

    my $ExperienceProfileId = $self->_getExperienceProfileID($PaymentMethodPayPalPlus);
    TestString('ExperienceProfileID', $ExperienceProfileId);

    my $TotalTaxPrice = $self->_getTotalTaxPrice($LineItemContainer);
    TestFloat('TotalTaxPrice', $TotalTaxPrice);

    my $BasketTotalAmount = $self->_getBasketTotalAmount($LineItemContainer);

    TestFloat('BasketTotalAmount', $BasketTotalAmount);

    my $ShippingPrice = $self->_getShippingPrice($LineItemContainer);
    TestFloat('ShippingPrice', $ShippingPrice);

    my $DiscountClass = $self->_createDiscount();
    my $DiscountPrice = $DiscountClass->getDiscountValue($LineItemContainer);
    TestFloat('DiscountPrice', $DiscountPrice);
    my $HandlingFeePrice = $DiscountClass->getFeeValue($LineItemContainer);
    TestFloat('HandlingFeePrice', $HandlingFeePrice);

    my $BasketCurrencyID = $self->_getBasketCurrencyID($LineItemContainer);
    TestString('BasketCurrencyID', $BasketCurrencyID);

    my $aLineItems = $self->_getBasketLineItems($LineItemContainer);
    my $aLineItemsDTO = $self->_createLineItemsRequestDTO($aLineItems, $BasketCurrencyID);
    my $aLineItemHashes = $self->_toHashLineItems($aLineItemsDTO);

    my $SubTotalPrice = $self->_calculateSubTotalPrice($aLineItemsDTO);

    # PayPal Payment-API-v1 does not support negative tax values as it may appear into epages basket,
    # hence we subtract from "subTotalPrice" and set "taxPrice" to zero, since "basketPrice" gotten
    # from "GrandTotal" value is the correct one.
    if ($TotalTaxPrice < 0) {
        $SubTotalPrice += $TotalTaxPrice;
        $TotalTaxPrice = 0;
        # provide single LineItem Sum Up to match with recalculated sub-total price
        $aLineItemHashes = $self->_sumUpHashLineItem($SubTotalPrice, $BasketCurrencyID);
    }

    my $LineItemPaymentGUID = $self->_getLineItemPaymentGUID($LineItemPaymentPayPalPlus);
    TestString('LineItemPaymentGUID', $LineItemPaymentGUID);

    my $Shop = $Basket->get('Shop');
    my $ShopUriBuilder = $self->_createShopUriBuilder($Shop);
    my $IPNCallbackUrl = $ShopUriBuilder->buildUri(PAYPALPLUS_IPN_CALLBACK_PATH);

    my $CreatePaymentRequestDTO = DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::Request->new();
    $CreatePaymentRequestDTO->experienceProfileId($ExperienceProfileId);
    $CreatePaymentRequestDTO->basketPrice($BasketTotalAmount);
    $CreatePaymentRequestDTO->subTotalPrice($SubTotalPrice);
    $CreatePaymentRequestDTO->shippingPrice($ShippingPrice);
    $CreatePaymentRequestDTO->discountPrice($DiscountPrice);
    $CreatePaymentRequestDTO->handlingFeePrice($HandlingFeePrice);
    $CreatePaymentRequestDTO->basketCurrency($BasketCurrencyID);
    $CreatePaymentRequestDTO->lineItems($aLineItemHashes);
    $CreatePaymentRequestDTO->lineItemPaymentGUID($LineItemPaymentGUID);
    $CreatePaymentRequestDTO->taxPrice($TotalTaxPrice);
    $CreatePaymentRequestDTO->notifyUrl($IPNCallbackUrl);

    $self->{'CreatePaymentRequestDTO'} = $CreatePaymentRequestDTO;
    return 1;
}


#========================================================================================
# function     getRequestDTO
# state        public
#----------------------------------------------------------------------------------------
# syntax       getRequestDTO( );
#----------------------------------------------------------------------------------------
# description  Get RequestDTO object
#----------------------------------------------------------------------------------------
# return      RequestDTO object | DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::Request
#========================================================================================
sub getRequestDTO {
    my $self = shift;
    return $self->{'CreatePaymentRequestDTO'};
}

sub _isNetShop {
    my $self = shift;
    my ($LineItemContainer) = @_;
    return $LineItemContainer->get('TaxModel') eq TAX_MODEL_NET;
}

sub _getTotalTaxAmount {
    my $self = shift;
    my ($LineItemContainer) = @_;
    return $LineItemContainer->get('TotalTax');
}

#========================================================================================
# function     _getBasketTotalAmount
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getBasketTotalAmount( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Get Basket value
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# return       Basket value | float
#========================================================================================
sub _getBasketTotalAmount {
    my $self = shift;
    my ($LineItemContainer) = @_;
    return $LineItemContainer->get('GrandTotal');
}

#========================================================================================
# function     _getTotalTaxPrice
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TaxValue = $self->_getTotalTaxPrice( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Get Basket line item tax price
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# return       $TaxValue | Basket line item tax value | float
#========================================================================================
sub _getTotalTaxPrice {
    my $self = shift;
    my ($LineItemContainer) = @_;

    if( $self->_isNetShop($LineItemContainer) ){
        return $self->_getTotalTaxAmount($LineItemContainer);
    } else {
        return $self->_getLineItemTaxPrice($LineItemContainer);
    }
}

#========================================================================================
# function     _getLineItemTaxPrice
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TaxValue = $self->_getLineItemTaxPrice( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Get Basket line item tax price
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# return       $TaxValue | Basket line item tax value | float
#========================================================================================
sub _getLineItemTaxPrice {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $aLineItemChildren = $LineItemContainer->children();
    my $LineItemTax;
    foreach my $LineItem (@{$aLineItemChildren}) {
        if ($LineItem->instanceOf('LineItemTax')) {
            $LineItemTax = $LineItem;
            last;
        }
    }

    return 0 unless defined($LineItemTax);
    return $LineItemTax->get('LineItemPrice');
}

#========================================================================================
# function     _getShippingPrice
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getShippingPrice( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Get shipping price
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# return       shipping price | float
#========================================================================================
sub _getShippingPrice {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $Shipping = $LineItemContainer->get('Shipping');
    if ( isValid($Shipping) ) {
        return $Shipping->get('LineItemPrice');
    } else{
        return 0;
    }
}

#========================================================================================
# function     _getExperienceProfileID
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getExperienceProfileID( $PaymentMethodPayPalPlus );
#----------------------------------------------------------------------------------------
# description  Get Experience profile id
#----------------------------------------------------------------------------------------
# input        $PaymentMethodPayPalPlus | PaymentMethodPayPalPlus | DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# return       Experience profile id | string
#========================================================================================
sub _getExperienceProfileID {
    my $self = shift;
    my ($PaymentMethodPayPalPlus) = @_;
    return $PaymentMethodPayPalPlus->get('ExperienceProfileId');
}

#========================================================================================
# function     _getBasketCurrencyID
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getBasketCurrencyID( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Get CurrencyID
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# return       Basket currency id | string
#========================================================================================
sub _getBasketCurrencyID {
    my $self = shift;
    my ($LineItemContainer) = @_;
    return $LineItemContainer->get('CurrencyID');
}

#========================================================================================
# function     _getLineItemPaymentGUID
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getLineItemPaymentGUID( $LineItemPaymentPayPalPlus );
#----------------------------------------------------------------------------------------
# description  Get lineItemPaymentGUID
#----------------------------------------------------------------------------------------
# input        $LineItemPaymentPayPalPlus | LineItemPaymentPayPalPlus | DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus
# return       lineItemPaymentGUID | string
#========================================================================================
sub _getLineItemPaymentGUID {
    my $self = shift;
    my ($LineItemPaymentPayPalPlus) = @_;
    return $LineItemPaymentPayPalPlus->get('GUID');
}

#========================================================================================
# function     _getBasketLineItems
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getBasketLineItems( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Get LineItems from the basket
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# return       LineItems | array with LineItems of the basket | array
#========================================================================================
sub _getBasketLineItems {
    my $self = shift;
    my ($LineItemContainer) = @_;
    return $LineItemContainer->get('LineItems');
}

#========================================================================================
# function     _createLineItemsRequestDTO
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createLineItemsRequestDTO( $aLineItems, $BasketCurrencyID );
#----------------------------------------------------------------------------------------
# description  create an array with the LineItemDTO objects
#----------------------------------------------------------------------------------------
# input        $aLineItems | array with LineItems of the basket | array
# input        $BasketCurrencyID | currency id of basket | integer
# return       LineItemsDTO | array with LineItemsDTO object | array
#========================================================================================
sub _createLineItemsRequestDTO {
    my $self = shift;
    my ($aLineItems , $BasketCurrencyID) = @_;
    my @LineItemsRequestDTO;
    foreach my $LineItem (@{$aLineItems}){
        my $LineItemRepository = $self->_createLineItemRepository();
        $LineItemRepository->collectData($LineItem);
        my $LineItemDTO = $LineItemRepository->getLineItemDTO();
        $LineItemDTO->currencyID($BasketCurrencyID);
        push(@LineItemsRequestDTO,$LineItemDTO);
    }
    return \@LineItemsRequestDTO;
}

#========================================================================================
# function     _toHashLineItems
# state        private
#----------------------------------------------------------------------------------------
# syntax       _toHashLineItems( $aLineItemsDTO );
#----------------------------------------------------------------------------------------
# description  create an array with LineItem Hashes
#----------------------------------------------------------------------------------------
# input       $aLineItemsDTO | array with LineItemsDTO object | ref array
# return      @LineItemsToHash | array with LineItems hashes | ref array
#========================================================================================
sub _toHashLineItems{
    my $self = shift;
    my ($aLineItemsDTO) = @_;
    my @LineItemsToHash;
    foreach my $LineItemDTO (@{$aLineItemsDTO}){
        push(@LineItemsToHash,$LineItemDTO->toHash());
    }
    return \@LineItemsToHash;
}

#========================================================================================
# function     _sumUpHashLineItem
# state        private
#----------------------------------------------------------------------------------------
# syntax       _sumUpHashLineItem( $SubTotalPrice, $BasketCurrencyID );
#----------------------------------------------------------------------------------------
# description  create an array with a single sum up LineItem Hash
#----------------------------------------------------------------------------------------
# input       $SubTotalPrice | basket sub-total price | int
# input       $BasketCurrencyID | currency ID string | string
# return      @lineItemSumUpHash | array with LineItem hash sum up | ref.array
#========================================================================================
sub _sumUpHashLineItem {
    my $self = shift;
    my ($SubTotalPrice, $BasketCurrencyID) = @_;

    return [{
        'name' => 'Line Item(s) Sum Up',
        'price' => $SubTotalPrice,
        'quantity' => 1,
        'currencyID' => $BasketCurrencyID,
    }];
}

sub _calculateSubTotalPrice {
    my $self = shift;
    my ($aLineItemsDTO) = @_;
    my $SubTotalPrice = 0;

    foreach my $LineItemsDTO (@{$aLineItemsDTO}){
        $SubTotalPrice += $LineItemsDTO->price() * $LineItemsDTO->quantity();
    }

    return $SubTotalPrice;
}
#========================================================================================
# function     _createLineItemRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createLineItemRepository( );
#----------------------------------------------------------------------------------------
# description  create LineItemRepository
#----------------------------------------------------------------------------------------
# return       LineItemRepository | DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::LineItem
#========================================================================================
sub _createLineItemRepository {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::LineItem->new();
}
sub _createDiscount {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Discount->new();
}
sub _createShopUriBuilder {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop);
}

1;