#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::CreatePayment_RepositoryRequest
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for CreatePayment::RequestDAO
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::CreatePayment_RepositoryRequest;
use base DE_EPAGES::Core::API::Test;


use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::LineItem;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::Request;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::LineItem;
use DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;

use Test::More;
use strict;

use constant TOTAL_TAX => 9.57;
use constant TAX_LINE_ITEM_PRICE_NEGATIVE => -5.10;
use constant SUB_TOTAL_PRICE => 54.90;

sub test {
    my $self = shift;

    $self->test_Successful_CollectData();
    $self->test_Successful_CollectData_NoShippingMethod();
    $self->test_Successful_CollectData_NetShop();
    $self->test_Successful_CollectData_TaxNegative();

    done_testing();
}

## no critic (ProhibitLongLines)
sub test_Successful_CollectData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_buildBasket_WithShippingMethod();

    my $CreatePayment_RepositoryRequest = $self->_createRequestDAO();
    ok($CreatePayment_RepositoryRequest->collectData($Basket), "$SubTestName - test if the collect is successful");
    my $CreatePaymentRequestDTO = $CreatePayment_RepositoryRequest->getRequestDTO();

    is_deeply($CreatePaymentRequestDTO->toHash() , $self->_expectedHash(), "$SubTestName - test if the toHash was build correctly");

    my $LineItems = $CreatePaymentRequestDTO->lineItems();
    my $LineItem1 = @{$LineItems}[0];
    is($LineItem1->{'currencyID'} , 'EUR', "$SubTestName - test if the currencyID was setted correctly at the first LineItemDTO");
    my $LineItem2 = @{$LineItems}[1];
    is($LineItem2->{'currencyID'} , 'EUR', "$SubTestName - test if the currencyID was setted correctly at the second LineItemDTO");
}

#----------------------------------------------------------------------------------------
sub test_Successful_CollectData_NoShippingMethod {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_buildBasket_WithoutShippingMethod();

    my $CreatePayment_RepositoryRequest = $self->_createRequestDAO();
    ok($CreatePayment_RepositoryRequest->collectData($Basket), "$SubTestName - test if the collect is successful");
    my $CreatePaymentRequestDTO = $CreatePayment_RepositoryRequest->getRequestDTO();

    is($CreatePaymentRequestDTO->shippingPrice() , '0', "$SubTestName - test if the shipping price was setted to 0 if there is no shippingMethod set.");
;
}
#----------------------------------------------------------------------------------------
sub test_Successful_CollectData_NetShop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_buildBasket_NetShop();

    my $CreatePayment_RepositoryRequest = $self->_createRequestDAO();
    ok($CreatePayment_RepositoryRequest->collectData($Basket), "$SubTestName - test if the collect is successful");
    my $CreatePaymentRequestDTO = $CreatePayment_RepositoryRequest->getRequestDTO();

    is($CreatePaymentRequestDTO->taxPrice() , TOTAL_TAX, "$SubTestName - test if the tax price was gotten from TotalTax attribute in case of net shop tax model");
}
#----------------------------------------------------------------------------------------
sub test_Successful_CollectData_TaxNegative {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_buildBasket_TaxNegative();

    my $CreatePayment_RepositoryRequest = $self->_createRequestDAO();
    ok($CreatePayment_RepositoryRequest->collectData($Basket), "$SubTestName - test if the collect is successful");
    my $CreatePaymentRequestDTO = $CreatePayment_RepositoryRequest->getRequestDTO();

    is($CreatePaymentRequestDTO->taxPrice() , 0, "$SubTestName - test if the tax price was set to zero in case of negative tax");
    is($CreatePaymentRequestDTO->subTotalPrice() , SUB_TOTAL_PRICE + TAX_LINE_ITEM_PRICE_NEGATIVE, "$SubTestName - test that subTotalPrice was decremented by negative tax amount");
}
#---------------------------------------------------------------------------
#  Mockify helpers
#---------------------------------------------------------------------------
sub _createRequestDAO {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Request', []);
    $Mockify->addMockWithReturnValue('_createLineItemRepository', $self->_createLineItemRepository());
    $Mockify->addMockWithReturnValue('_createDiscount', $self->_createDiscount());
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder(), [{'object' => 'DE_EPAGES::Shop::API::Object::Shop'}]);

    return $Mockify->getMockObject();
}
#---------------------------------------------------------------------------
sub _createDiscount {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::Discount', []);
    $Mockify->addMockWithReturnValueAndParameterCheck('getDiscountValue', -10, [{ 'object' => 'DE_EPAGES::Order::API::Object::LineItemContainer' }]);
    $Mockify->addMockWithReturnValueAndParameterCheck('getFeeValue', 5, [{ 'object' => 'DE_EPAGES::Order::API::Object::LineItemContainer' }]);
    return $Mockify->getMockObject();
}
#---------------------------------------------------------------------------
sub _createLineItemRepository {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::Repository::LineItem', []);
    $Mockify->addMockWithReturnValueAndParameterCheck('collectData', undef, [{'object' => 'DE_EPAGES::Product::API::Object::LineItemProduct'}]);

    my $LineItemDTO  = DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::LineItem->new();
    $LineItemDTO->price('9.15');
    $LineItemDTO->quantity('3');
    $Mockify->addMockWithReturnValue('getLineItemDTO', $LineItemDTO);

    return $Mockify->getMockObject();
}
#---------------------------------------------------------------------------
#  Helpers
#---------------------------------------------------------------------------
sub _buildBasket_WithShippingMethod {
    my $self = shift;
    my $IsGrossShop = 1;
    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer($IsGrossShop),
    };
    my $Basket = $self->_createBasket($hAttributes);
    return $Basket;
}
#---------------------------------------------------------------------------
sub _buildBasket_WithoutShippingMethod {
    my $self = shift;
    my $IsGrossShop = 1;
    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer_withoutShippingMethod($IsGrossShop),
    };
    my $Basket = $self->_createBasket($hAttributes);
    return $Basket;
}
sub _buildBasket_NetShop {
    my $self = shift;
    my $IsGrossShop = 0;
    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer($IsGrossShop),
    };
    my $Basket = $self->_createBasket($hAttributes);
    return $Basket;
}
sub _buildBasket_TaxNegative {
    my $self = shift;
    my $IsGrossShop = 1;
    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer_withLineItemTaxNegative($IsGrossShop),
    };
    my $Basket = $self->_createBasket($hAttributes);
    return $Basket;
}
#---------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;
    my ($hAdditionalAttributes) = @_;
    my $hAttributes = {
        'Alias' => 'Basket',
        'Shop' => $self->_createShop(),
        %{$hAdditionalAttributes},
    };
    my $Basket = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', $hAttributes );
    return $Basket->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'RESTAPIURL' => 'https://publicEpages/base/url/',
        'IsInternalTestShop' => '0',
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'EPagesJBaseURL' => 'http://localhost:8088/rs/ '
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#---------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;
    my ($IsGrossShop) = @_;

    my $LineItemProducts = $self->_createLineItemProducts();
    my $LineItemShipping = $self->_createLineItemShipping();

    my $hAttributes = {
        'Alias' => 'MyContainer',
        'GrandTotal' => 75.57,
        'TotalTax' => TOTAL_TAX,
        'CurrencyID' => 'EUR',
        'LineItems' => $LineItemProducts,
        'Payment' => $self->_createLineItemPaymentPayPalPlus(),
        'Shipping' => $LineItemShipping,
        'TaxModel' => $IsGrossShop,
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );
    $LineItemContainer->addMockWithReturnValue('children', [@$LineItemProducts, $LineItemShipping]);

    return $LineItemContainer->getMockObject();
}
#---------------------------------------------------------------------------
sub _createLineItemContainer_withoutShippingMethod {
    my $self = shift;
    my ($IsGrossShop) = @_;

    my $LineItemProducts = $self->_createLineItemProducts();
    my $hAttributes = {
        'Alias' => 'MyContainer',
        'GrandTotal' => 80.57,
        'CurrencyID' => 'EUR',
        'LineItems' => $LineItemProducts,
        'Payment' => $self->_createLineItemPaymentPayPalPlus(),
        'Shipping' => undef,
        'TaxModel' => $IsGrossShop,
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );
    $LineItemContainer->addMockWithReturnValue('children', [@$LineItemProducts]);

    return $LineItemContainer->getMockObject();
}
#---------------------------------------------------------------------------
sub _createLineItemContainer_withLineItemTaxNegative {
    my $self = shift;
    my ($IsGrossShop) = @_;

    my $LineItemProducts = $self->_createLineItemProducts();
    my $LineItemShipping = $self->_createLineItemShipping();
    my $LineItemTax = $self->_createLineItemTaxNegative();

    my $hAttributes = {
        'Alias' => 'MyContainer',
        'GrandTotal' => 39.45,
        'TotalTax' => 0,
        'CurrencyID' => 'EUR',
        'LineItems' => $LineItemProducts,
        'Payment' => $self->_createLineItemPaymentPayPalPlus(),
        'Shipping' => $LineItemShipping,
        'TaxModel' => $IsGrossShop,
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );
    $LineItemContainer->addMockWithReturnValue('children', [@$LineItemProducts, $LineItemShipping, $LineItemTax]);

    return $LineItemContainer->getMockObject();
}

#---------------------------------------------------------------------------
sub _createLineItemProduct {
    my $self = shift;
    my ($ProductAlias) = @_;

    my $LineItem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Product::API::Object::LineItemProduct',
            { 'Alias' => $ProductAlias }
    );
    $LineItem->addMock('instanceOf', sub {
            my $self = shift;
            my ($ClassName) = @_;
            return $ClassName =~ /LineItemProduct$/ ? 1 : 0;
    });

    return $LineItem->getMockObject()
}
#---------------------------------------------------------------------------
sub _createLineItemProducts {
    my $self = shift;

     my $LineItem1 = $self->_createLineItemProduct('MyProduct1');
     my $LineItem2 = $self->_createLineItemProduct('MyProduct2');

    return [$LineItem1, $LineItem2];
}
#---------------------------------------------------------------------------
sub _createLineItemPaymentPayPalPlus {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'LineItemPaymentPayPalPlus',
        'GUID' => 'myGUID',
        'PaymentMethod' => $self->_createPaymentMethodPayPalPlus(),
    };
    my $LineItemPaymentPayPalPlus = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus', $hAttributes );
    return $LineItemPaymentPayPalPlus->getMockObject();
}
#---------------------------------------------------------------------------
sub _createLineItemShipping {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'LineItemShipping',
        'LineItemPrice' => 25.67,
    };
    my $LineItemShipping = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemShipping', $hAttributes );
        $LineItemShipping->addMock('instanceOf', sub {
            my $self = shift;
            my ($ClassName) = @_;
            return $ClassName =~ /LineItemShipping$/ ? 1 : 0;
    });

    return $LineItemShipping->getMockObject();
}
#---------------------------------------------------------------------------
sub _createLineItemTaxNegative {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'LineItemTax',
        'LineItemPrice' => TAX_LINE_ITEM_PRICE_NEGATIVE,
    };
    my $LineItemTax = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemTax', $hAttributes );
    $LineItemTax->addMock('instanceOf', sub {
        my $self = shift;
        my ($ClassName) = @_;
        return $ClassName =~ /LineItemTax$/ ? 1 : 0;
    });

    return $LineItemTax->getMockObject();
}
#---------------------------------------------------------------------------
sub _createPaymentMethodPayPalPlus {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'PaymentMethodPayPalPlus',
        'ExperienceProfileId' => 'myProfileID',
        'IsInTestMode' => 1,
        'MerchantClientId' => 'MyDummyMerchantClientId',
        'PaymentType' => $self->_createPaymentType(),
        'Shop' => $self->_createShop(),
    };
    my $PaymentMethodPayPalPlus = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus', $hAttributes );
    return $PaymentMethodPayPalPlus->getMockObject();
}
#---------------------------------------------------------------------------
sub _createPaymentType {
    my $self = shift;

    my $hAttributes = {
        'ProviderClientId' => 'MyDummyProviderClientId',
        'ProviderClientSecret' => 'MyDummyProviderClientSecret',
        'PayPalPartnerAttributionId' => 'MyAttributionId',
    };
    my $PaymentType = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Payment::API::Object::PaymentType', $hAttributes );

    return $PaymentType->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShopUriBuilder {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Shop::API::ShopUriBuilder',
        [$self->_createShop(), $self->_createSystem()]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('buildUri', 
        'https://epagesj/base/url/shops/TestShop/payments/paypalplus/notification', 
        [{'string' => 'payments/paypalplus/notification'}]
    );

    return $Mockify->getMockObject();
}
#---------------------------------------------------------------------------
#  Expected results
#---------------------------------------------------------------------------
sub _expectedHash {
    my $self = shift;
    my $hRequest = {
          'basketCurrency' => 'EUR',
          'returnUrl' => undef,
          'basketPrice' => '75.57',
          'subTotalPrice' => SUB_TOTAL_PRICE,
          'taxPrice' => '0',
          'shippingPrice' => '25.67',
          'discountPrice' => '-10',
          'handlingFeePrice' => '5',
          'experienceProfileId' => 'myProfileID',
          'lineItems' => [
                           {
                             'currencyID' => 'EUR',
                             'quantity' => '3',
                             'name' => undef,
                             'price' => '9.15',
                           },
                           {
                             'currencyID' => 'EUR',
                             'quantity' => '3',
                             'name' => undef,
                             'price' => '9.15',
                           }
                         ],
          'lineItemPaymentGUID' => 'myGUID',
          'cancelUrl' => undef,
          'notifyUrl' => 'https://epagesj/base/url/shops/TestShop/payments/paypalplus/notification'
    };
    return $hRequest;
}


## use critic (ProhibitLongLines)
__PACKAGE__->runScript();