#========================================================================================
# package      DE_EPAGES::Saferpay2::API::SaferpayService
# state        public
#----------------------------------------------------------------------------------------
# Saferpay Service
#========================================================================================

package DE_EPAGES::Saferpay2::API::SaferpayService;


use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Saferpay2::API::Constants qw (
    SAFERPAY_SPEC_VERSION
    PAYMENT_TYPE_SAFERPAY2
    SUPPORTED_PAYMENT_METHODS_MAP
    PAYMENT_METHOD_KLARNA
);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::PerlTools qw (GenerateGUID);
use DE_EPAGES::Saferpay2::API::Tools qw ( FormatFloatForSaferpay );
use DE_EPAGES::Saferpay2::API::InitializePaymentPageMapper;

use HTTP::Request;
use HTTP::Status qw ( is_server_error );
use JSON::MaybeXS qw ( encode_json decode_json );
use LWP::UserAgent;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Saferpay2::API::SaferpayService->new( %args );
#----------------------------------------------------------------------------------------
# description  Instantiates a new SaferpayService.
#----------------------------------------------------------------------------------------
# input        $PaymentMethod | PaymentMethod object | DE_EPAGES::Saferpay2::API::Object::PaymentMethodSaferpay2
# input        $hCustomValues | hash with custom values to override PaymentMethod | hashref
# return       SaferpayService | object | DE_EPAGES::Saferpay2::API::SaferpayService
#========================================================================================
sub new {
    my $class = shift;
    my ( $LineItemPayment, $hCustomValues ) = @_;

    my $PaymentMethod = $LineItemPayment ? $LineItemPayment->get('PaymentMethod') : undef;

    # this gets overridden with custom values,
    # it is used in the  settings page to check form values before save
    my $self  = bless {
        MerchantName     => $hCustomValues->{'MerchantName'} // $PaymentMethod->get('MerchantName'),
        MerchantPassword => $hCustomValues->{'MerchantPassword'} // $PaymentMethod->get('MerchantPassword'),
        CustomerId       => $hCustomValues->{'CustomerId'} // $PaymentMethod->get('CustomerId'),
        TerminalId       => $hCustomValues->{'TerminalId'} // $PaymentMethod->get('TerminalId'),
        IsInTestMode     => $hCustomValues->{'IsInTestMode'} // $LineItemPayment->get('IsTestTransaction'),
    }, $class;

    return $self;
}


#========================================================================================
# function     InitializePaymentPage
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Saferpay2::API::SaferpayService->InitializePaymentPage( $PaymentWebhookUrl,
# $ShopperRedirectUrl );
#----------------------------------------------------------------------------------------
# description  With this request, you forward all data necessary for the payment to Saferpay.
#----------------------------------------------------------------------------------------
# input        $Order  | Order object  | object
# input        $PaymentWebhookUrl  | NotifyUrl to notify the merchant system, as a redundant measure  | string
# input        $ShopperRedirectUrl | request object to handle | string
# return       Response of PaymentPage Initialize | hashref
#========================================================================================
sub InitializePaymentPage {
    my $self    = shift;
    my ($Order, $PaymentWebhookUrl, $ShopperRedirectUrl) = @_;

    my $System = LoadRootObject();
    my $ApiBaseUrl = $self->_getBaseUrl($System);

    my $LineItemContainer = $Order->get('LineItemContainer');

    my $Shop = $Order->getSite();
    my $Url = $ApiBaseUrl."Payment/v1/PaymentPage/Initialize";
    my $Header = ['Content-Type' => 'application/json; charset=UTF-8'];
    my $hContainerInfo = $LineItemContainer->get([ qw( CurrencyID GrandTotal )]);

    my $SplitGatewayMethod = $LineItemContainer->get('Payment')->get('PaymentMethod')->get('SplitGatewayMethod');
    my $OrderId = $Order->alias();
    $OrderId =~ s/[\W]/-/g;

    my $hData = {
        RequestHeader => {
            SpecVersion    => SAFERPAY_SPEC_VERSION,
            CustomerId     => $self->{'CustomerId'},
            RequestId      => GenerateGUID(),
            RetryIndicator => 0,
            ClientInfo     => $self->_getShopInfo()
        },
        TerminalId    => $self->{'TerminalId'},
        Payment       => {
            Amount      => {
                Value        => FormatFloatForSaferpay($hContainerInfo->{'GrandTotal'}, $hContainerInfo->{'CurrencyID'}),
                CurrencyCode => $hContainerInfo->{'CurrencyID'}
            },
            OrderId     => $OrderId,
            Description => $Shop->get('NameOrAlias')
        },
        %{SUPPORTED_PAYMENT_METHODS_MAP->{$SplitGatewayMethod}},
        ReturnUrls    => {
            Success => $ShopperRedirectUrl,
            Fail    => $ShopperRedirectUrl
        },
        Notification  => {
            NotifyUrl => $PaymentWebhookUrl
        },
        RequestURL    => $Url
    };

    if($SplitGatewayMethod eq  PAYMENT_METHOD_KLARNA) {
        eval {
            my $Mapper = DE_EPAGES::Saferpay2::API::InitializePaymentPageMapper->new($hData, $LineItemContainer);
            $hData = $Mapper->MapAdditionalKlarnaFields();
        };
        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);
        }
    }

    $self->_LogPayment('InitializePaymentPage Request', {'request' => $hData});
    my $hResponse = $self->_doPostRequest($Url, $Header, encode_json($hData));
    $self->_LogPayment('InitializePaymentPage Response', {'response' => $hResponse});

    return $hResponse;
}


sub DoAssertCall {
    my $self = shift;
    my ($Token, $Process) = @_;

    my $System = LoadRootObject();
    my $ApiBaseUrl = $self->_getBaseUrl($System);

    my $Url = $ApiBaseUrl . "Payment/v1/PaymentPage/Assert";
    my $Header = [ 'Content-Type' => 'application/json; charset=UTF-8' ];

    my $hData = {
        RequestHeader => {
            SpecVersion    => SAFERPAY_SPEC_VERSION,
            CustomerId     => $self->{'CustomerId'},
            RequestId      => GenerateGUID(),
            RetryIndicator => 0,
            ClientInfo     => $self->_getShopInfo()
        },
        Token         => $Token,
        RequestURL    => $Url
    };

    $self->_LogPayment('DoAssertCall Request', { 'request' => $hData, 'process' => $Process });
    my $hResponse = $self->_doPostRequest($Url, $Header, encode_json($hData));
    $self->_LogPayment('DoAssertCall Response', { 'response' => $hResponse });
    return $hResponse;
}

sub CaptureTransaction {
    my $self = shift;
    my ($TransID) = @_;

    my $System = LoadRootObject();
    my $ApiBaseUrl = $self->_getBaseUrl($System);
    my $Url = $ApiBaseUrl . "/Payment/v1/Transaction/Capture";
    my $Header = [ 'Content-Type' => 'application/json; charset=UTF-8' ];

    my $hData = {
        RequestHeader => {
            SpecVersion    => SAFERPAY_SPEC_VERSION,
            CustomerId     => $self->{'CustomerId'},
            RequestId      => GenerateGUID(),
            RetryIndicator => 0,
            ClientInfo     => $self->_getShopInfo()
        },
        TransactionReference => {
            TransactionId => $TransID,
        },
        RequestURL    => $Url
    };

    $self->_LogPayment('CaptureTransaction Request', { 'request' => $hData });
    my $hResponse = $self->_doPostRequest($Url, $Header, encode_json($hData));
    $self->_LogPayment('CaptureTransaction Response', { 'response' => $hResponse });
    return $hResponse;
}


sub _doPostRequest {
    my $self = shift;
    my ($Url, $Header, $Data) = @_;

    my $request = HTTP::Request->new('POST', $Url, $Header, $Data);
    $request->authorization_basic($self->{'MerchantName'}, $self->{'MerchantPassword'});
    my $userAgent = LWP::UserAgent->new();

    my $res = $userAgent->request($request);
    my $hResponse;
    if ( is_server_error($res->code) ) {
        $hResponse = {
            Behavior     => 'ABORT',
            ErrorMessage => 'Internal Server Error',
            ErrorName => 'INTERNAL_SERVER_ERROR'
        }
    } else {
        $hResponse = decode_json($res->decoded_content());
    }
    return $hResponse;
}

sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_SAFERPAY2, $Name, $DebugData, $Section, $Filter);
}

sub _getBaseUrl {
    my $self = shift;
    my ($System) = @_;

    my $ApiBaseUrl;
    if ($self->{'IsInTestMode'}) {
        $ApiBaseUrl = $System->get('Saferpay2_WebserviceEndpointURL_test');
    } else{
        $ApiBaseUrl = $System->get('Saferpay2_WebserviceEndpointURL_live');
    }

    return $ApiBaseUrl;
}
sub _getShopInfo {
    my $self = shift;
    return {
        'ShopInfo' => 'EPAGESNOW_' . $self->_getEpagesVersion().':EPAGES_1.0.0',
        'OsInfo'   => $self->_getOsInfo()
    };
}

sub _getEpagesVersion {
    my $self = shift;
    return LoadRootObject()->get('EpagesVersion');
}

sub _getOsInfo {
    my $self = shift;
    return $^O;
}

1;