#========================================================================================
# package      DE_EPAGES::Shop::API::Resource::ShopRequestContext
# state        public
# description  helper class for shop-dependend REST resources.
#========================================================================================
package DE_EPAGES::Shop::API::Resource::ShopRequestContext;

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::TestParameter qw (TestObject);
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);

use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopRequestContext = $DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
#----------------------------------------------------------------------------------------
# description  Create a new ShopRequestContext object for a given shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGET::Shop::API::Object::Shop
# return       $ShopRequestContext | request context | DE_EPAGES::Shop::API::Resource::ShopRequestContext
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop) = @_;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    return bless {
        Shop => $Shop,
    }, $class;
}



#========================================================================================
# function     shop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Shop = $ShopRequestContext->shop;
#----------------------------------------------------------------------------------------
# description  Returns the shop.
#----------------------------------------------------------------------------------------
# return       $Shop | shop of the current request | DE_EPAGET::Shop::API::Object::Shop
#========================================================================================
sub shop {
    return shift->{'Shop'};
}



#========================================================================================
# function     validCurrency
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CurrencyID = $ShopRequestContext->validCurrency($RequestCurrencyID);
#----------------------------------------------------------------------------------------
# description  Returns the default shop locale if $RequestCurrencyID is undefined.
#               Returns $RequestCurrencyID if the currency is supported by the shop.
#               Throws a BadRequest error if the currency is not supported by the shop.
#----------------------------------------------------------------------------------------
# input        $RequestCurrencyID | currency id, e.g. 'EUR' | string
# return       $CurrencyID | currency id, e.g. 'EUR' | string
#========================================================================================
sub validCurrency {
    my $self = shift;
    my ($RequestCurrencyID) = @_;

    my $Shop = $self->shop;
    return $Shop->get('CurrencyID') unless defined $RequestCurrencyID;

    Error('BadRequest', {
        'InvalidCurrency' => $RequestCurrencyID,
        'SupportedCurrencies' => join(',', @{ $Shop->currencies }),
        'ShopAlias' => $Shop->alias(),
    }) unless $Shop->existsCurrency($RequestCurrencyID);

    return $RequestCurrencyID;
}

#========================================================================================
# function     unityFacetedSearchShowFacetsOnCategory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShowOrNot = $ShopRequestContext->unityFacetedSearchShowFacetsOnCategory();
#----------------------------------------------------------------------------------------
# description  Returns if the faceted search will be shown on category pages in now shops.
#               Depends on both attributes, 'FacetedSearchShowFacets' and 'UnityFacetedSearchShowFacetsOnCategory'
#----------------------------------------------------------------------------------------
# input        -
# return       $ShowOrNot | Whether to show the faceted search on category pages or not | boolean
#========================================================================================
sub unityFacetedSearchShowFacetsOnCategory {
    my $self = shift;
    return $self->shop->get('FacetedSearchShowFacets') && $self->shop->get('UnityFacetedSearchShowFacetsOnCategory') ? 1 : 0;
}

#========================================================================================
# function     facetedSearchStartDepth
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CurrencyID = $ShopRequestContext->facetedSearchStartDepth();
#----------------------------------------------------------------------------------------
# description  Returns the category start depth for faceted search
#----------------------------------------------------------------------------------------
# input        -
# return       $FacetedSearchStartDepth | category start depth for faceted search | int
#========================================================================================
sub facetedSearchStartDepth {
    my $self = shift;
    return $self->shop->get('FacetedSearchStartDepth');
}

#========================================================================================
# function     validLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LocaleID = $ShopRequestContext->validLocale($RequestLocaleID);
#----------------------------------------------------------------------------------------
# description  Returns the default shop locale if $RequestLocaleID is undefined.
#               Returns $RequestLocaleID if the locale is supported by the shop.
#               Throws a BadRequest error if the locale is not supported by the shop.
#----------------------------------------------------------------------------------------
# input        $RequestLocaleID | locale id, e.g. 'de_DE' | string
# return       $LocaleID | locale id, e.g. 'de_DE' | string
#========================================================================================
sub validLocale {
    my $self = shift;
    my ($LocaleID) = @_;

    my $Shop = $self->shop;
    return $Shop->get('LocaleID') unless defined $LocaleID;
    Error('BadRequest', {
        'InvalidLocale' => $LocaleID,
        'SupportedLocales' => join(',', map { $_->get('LocaleID') } @{ $Shop->get('SiteLocales') }),
        'ShopAlias' => $Shop->alias(),
    }) unless $Shop->existsSiteLocale($LocaleID);
    return $LocaleID;
}

#========================================================================================
# function     _apiBaseUrl
# state        private
#----------------------------------------------------------------------------------------
# syntax       $BaseURL = $ShopRequestContext->_apiBaseUrl;
#----------------------------------------------------------------------------------------
# description  Returns the base URL for the REST API of the shop.
#----------------------------------------------------------------------------------------
# return       $BaseURL | REST API base URL, e.g. 'https://shop-domain' | string
#========================================================================================
sub _apiBaseUrl {
    my $self = shift;

    my $Shop = $self->shop;
    my $System = LoadRootObject();
    if ($Shop->get('DomainName') && ($Shop->get('HasSSLCertificate') || !$System->get('HasSSLCertificate'))) {
        return $Shop->get('ProtocolAndServerSSL');
    }
    else { # EPG-34475: workaround for sites without domain name
        my $SystemBaseURL = $System->get('ProtocolAndServerSSL');
        my $ShortUrl = $Shop->get('WebServerScriptNamePart');
        if ($ShortUrl) {
            return "$SystemBaseURL/epages/$ShortUrl.rest"
        }
        else {
            my $StoreName = $System->get('StoreName');
            my $ShopGUID = $Shop->get('GUID');
            return "$SystemBaseURL/epages/$StoreName.rest/shops/$ShopGUID"
        }
    }
}



#========================================================================================
# function     apiBaseUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $BaseURL = $ShopRequestContext->apiBaseUrl;
#----------------------------------------------------------------------------------------
# description  Returns the base URL for the REST API of the shop.
#----------------------------------------------------------------------------------------
# return       $BaseURL | REST API base URL, e.g. 'https://shop-domain' | string
#========================================================================================
sub apiBaseUrl {
    my $self = shift;
    return $self->{'apiBaseUrl'} //= $self->_apiBaseUrl;
}



#========================================================================================
# function     imageBaseUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $BaseURL = $ShopRequestContext->imageBaseUrl;
#----------------------------------------------------------------------------------------
# description  Returns the base URL for images of the shop.
#----------------------------------------------------------------------------------------
# return       $BaseURL | REST API base URL, e.g. 'https://shop-domain' | string
#========================================================================================
sub imageBaseUrl {
    my $self = shift;
    return $self->{'imageBaseUrl'} //= $self->shop->get('ProtocolAndServerSSL');
}

1;