#========================================================================================
# package      DE_EPAGES::ShopMigration::API::CrossSellingsCommon
# state        public
#----------------------------------------------------------------------------------------
# description  common functions for exporting/importing cross sellings
#========================================================================================
package DE_EPAGES::ShopMigration::API::CrossSellingsCommon;
use base qw(Exporter);

use strict;
our @EXPORT_OK = qw(
    GetCrossSellings
    SetCrossSellings
    GetCrossSellingData
    SetCrossSellingData
    GetCrossSellingIDs
);

use DE_EPAGES::Core::API::Error qw(Error);
use DE_EPAGES::Core::API::TestParameter qw(TestString);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw(LoadClassByAlias LoadObject);
use DE_EPAGES::CrossSelling::API::CrossSelling qw( FindCrossSelling );  ##no critic (RequireCorrectImports)
use DE_EPAGES::ShopMigration::API::Constants qw(CROSSSELLINGS_ATTRIBUTES CROSSSELLINGS_OBJECTATTRIBUTES);

#========================================================================================
# function     GetCrossSellings
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahCrossSellings = GetCrossSellings($Shop);
#----------------------------------------------------------------------------------------
# description  Fetches all crosssellings
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
#----------------------------------------------------------------------------------------
# return       $ahCrossSellings | see GetCrossSellingData() | reference to array of hashes
#========================================================================================
sub GetCrossSellings {
    my ($Shop) = @_;

    my $Class = LoadClassByAlias('CrossSelling');
    my @Items;
    my $aCrossSellingIDs = GetCrossSellingIDs($Shop);
    foreach my $CrossSellingID (@$aCrossSellingIDs) {
        my $CrossSelling = $Class->loadObject($CrossSellingID);
        my $hInfo = GetCrossSellingData($Shop, $CrossSelling);
        push @Items, $hInfo if (keys %$hInfo);
    }
    return \@Items;
}

#========================================================================================
# function     GetCrossSellingIDs
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahCrossSellings = GetCrossSellingIDs($Shop);
#----------------------------------------------------------------------------------------
# description  Fetches all crosssellings as a list of IDs
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
#----------------------------------------------------------------------------------------
# return       $aItems | list of CrossSelling object IDs | ref array integer
#========================================================================================
sub GetCrossSellingIDs {
    my ($Shop) = @_;

    my $Items;
    my $dbi = GetCurrentDBHandle();
    my $Query = <<SQL;
        SELECT cs.crosssellingid
        FROM crossselling cs JOIN product p on cs.productid = p.productid
                             JOIN product t on cs.targetproductid = t.productid
                             LEFT JOIN product sp ON sp.productid = p.superproductid
                             LEFT JOIN product st ON st.productid = t.superproductid
        WHERE p.shopid = ?
          AND t.shopid = ?
          AND NOT EXISTS ( SELECT b.bundleproductid FROM bundleproductmap b WHERE b.bundleproductid=p.productid )
          AND NOT EXISTS ( SELECT b.bundleproductid FROM bundleproductmap b WHERE b.bundleproductid=t.productid )
          AND NOT EXISTS ( SELECT 1 FROM product pmaster WHERE pmaster.productid = p.superproductid AND pmaster.iscustomizable = 1 )
          AND NOT EXISTS ( SELECT 1 FROM product tmaster WHERE tmaster.productid = t.superproductid AND tmaster.iscustomizable = 1 )
          AND p.iscustomizable = 0
          AND p.isdownloadproduct = 0
          AND p.isappointment = 0
          AND t.iscustomizable = 0
          AND t.isdownloadproduct = 0
          AND t.isappointment = 0
          AND IFNULL(sp.iscustomizable,0)=0
          AND IFNULL(st.iscustomizable,0)=0
        ORDER BY cs.crosssellingid ASC
SQL
    my $aaResult = $dbi->execute($Query, [ $Shop->id, $Shop->id ], ['Integer']);
    push @$Items, map { $_->[0] } @{$aaResult};
    return $Items;
}

#========================================================================================
# function     GetCrossSellingData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hCrossSelling = GetCrossSellingData($Shop, $CrossSelling);
#----------------------------------------------------------------------------------------
# description  Constructs a hash with the exportable fields for a single CrossSelling
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $CrossSelling | crossselling | object
#----------------------------------------------------------------------------------------
# return       $hCrossSelling | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>Product                         - string (path of product)</li>
#                           <li>TargetProduct                   - string (path of target product)</li>
#                           <li>Position                        - integer</li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub GetCrossSellingData {
    my ($Shop, $CrossSelling) = @_;

    my $Attributes = [ @{CROSSSELLINGS_ATTRIBUTES()} ];
    my $ObjectAttributes = [ @{CROSSSELLINGS_OBJECTATTRIBUTES()} ];
    my $hValues = { Attributes => $CrossSelling->get($Attributes) };
    foreach my $AttributeName (@$ObjectAttributes) {
        my $Value = $hValues->{Attributes}{$AttributeName};
        next unless defined $Value;
        $hValues->{Attributes}{$AttributeName} = $Shop->relativePath($Value);
    }
    return $hValues;
}

#========================================================================================
# function     SetCrossSellings
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetCrossSellings($Shop, $ahCrossSellings);
#----------------------------------------------------------------------------------------
# description  sets attributes on CrossSellings as specified in the $ahCrossSellings
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $ahCrossSellings | see GetCrossSellings() | reference to array of hashes
#========================================================================================
sub SetCrossSellings {
    my ($Shop, $ahCrossSellings) = @_;

    foreach my $hCrossSelling (@$ahCrossSellings) {
        SetCrossSellingData($Shop, $hCrossSelling);
    }
}

#========================================================================================
# function     SetCrossSellingData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetCrossSellingData($Shop, $hCrossSelling);
#----------------------------------------------------------------------------------------
# description  inserts or sets attributes on a CrossSelling as specified in the hash $hCrossSelling
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $hCrossSelling | see GetCrossSellings() | reference to array of hashes
#========================================================================================
sub SetCrossSellingData {
    my ($Shop, $hCrossSelling) = @_;

    _analyzeAttributesHash($Shop, $hCrossSelling);
    my $CrossSelling = $hCrossSelling->{'CrossSelling'};
    if (defined $CrossSelling) {
        $CrossSelling->set($hCrossSelling->{'Attributes'});
    } else {
        #insert:
        my $Product = $hCrossSelling->{'Attributes'}{'Product'};
        my $Folder = $Product->folder('CrossSelling');
        $Folder->insertChild('CrossSelling', {
            Alias => $Folder->nextAlias,
            %{$hCrossSelling->{'Attributes'}},
        });
    }
    return;
}

#========================================================================================
# function     _analyzeAttributesHash
# state        private
#----------------------------------------------------------------------------------------
# syntax       _analyzeAttributesHash($hLineData);
#----------------------------------------------------------------------------------------
# description  Analyzes the fields in the parsed $hLineData
#               converts pathes to objects
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $hLineData | Hash with Values | ref hash string
#========================================================================================
sub _analyzeAttributesHash {
    my ($Shop, $hLineData) = @_;

    my ($Product, $TargetProduct);
    my $ProductPath = $hLineData->{'Attributes'}{'Product'};
    my $TargetProductPath = $hLineData->{'Attributes'}{'TargetProduct'};;
    $ProductPath =~ s/\s+$|^\s+//g;
    if ($ProductPath eq '') {
        Error('CROSSSELLING_EMPTY_PRODUCT', { 'Attribute' => 'Product', 'Value' => $ProductPath, 'TargetProduct' => $TargetProductPath });
    } else {
        $Product = _loadShopObject($Shop, $ProductPath, 'Product');
        $hLineData->{'Attributes'}{'Product'} = $Product;
    }

    $TargetProductPath =~ s/\s+$|^\s+//g;
    if ($TargetProductPath eq '') {
        Error('CROSSSELLING_EMPTY_TARGETPRODUCT', { 'Attribute' => 'TargetProduct', 'Value' => $TargetProductPath, 'Product' => $ProductPath });
    } else {
        $TargetProduct = _loadShopObject($Shop, $TargetProductPath, 'Product');
        $hLineData->{'Attributes'}{'TargetProduct'} = $TargetProduct;
    }

    # exists cross selling?
    my $CrossSelling = FindCrossSelling($Product, $TargetProduct, 'CrossSelling');
    $hLineData->{'CrossSellingClass'} = LoadClassByAlias('CrossSelling');
    $hLineData->{'CrossSelling'} = $CrossSelling;
    return;
}

#========================================================================================
# function     _loadShopObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = _loadShopObject( $ParentObject, $Path, $ClassAlias )
#----------------------------------------------------------------------------------------
# description  loads the object specified by object path and verifies that it exists,
#               belongs to the given parent object and is an instance of a certain class.
#               Returns undef if path doesn't exist or object doesn't belong to the same shop.
#               Returns the object if all tests passed.
#----------------------------------------------------------------------------------------
# input        $ParentObject | parent object | shop object
# input        $Path | object path to load | string
# input        $ClassAlias | alias of a class (optional) | string
# return       $Object | object | object
#========================================================================================
sub _loadShopObject {
    my ($ParentObject, $Path, $ClassAlias) = @_;

    TestString('Path', $Path);
    my $ObjectID = $ParentObject->findPath($Path);
    Error('UNKNOWN_OBJECT' => {
        'Value' => $Path,
    }) unless defined $ObjectID;

    # get object
    my $ObjectByPath = LoadObject($ObjectID);
    # check if object belongs to the current shop
    Error('FORBIDDEN_OBJECT' => {
        'Value' => $Path,
        'Class' => $ClassAlias,
    }) if (!_checkShopObject($ParentObject->getSite(), $ObjectByPath, $ClassAlias));
    return $ObjectByPath;
}

#========================================================================================
# function     _checkShopObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       _checkShopObject( $Shop, $Object, $ClassAlias )
#----------------------------------------------------------------------------------------
# description  Checks if an object belongs to a site (shop) and
#               is instance of a certain class. Returns 0 if not.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $Object | test object | object
# input        $ClassAlias | alias of a class (optional) | string
# return       $OK | true if check was ok | boolean
#========================================================================================
sub _checkShopObject {
    my ($Shop, $Object, $ClassAlias) = @_;

    if ($Object->getSite()->id != $Shop->id) {
        return 0;
    }
    if ($ClassAlias and not $Object->instanceOf($ClassAlias)) {
        return 0;
    }
    return 1;
}

1;
