#========================================================================================
# package      DE_EPAGES::ShopMigration::API::PriceListScalePricesCommon
# state        public
#----------------------------------------------------------------------------------------
# description common functions for exporting/importing PriceListScalePrices
#========================================================================================
package DE_EPAGES::ShopMigration::API::PriceListScalePricesCommon;
use base qw(Exporter);

use strict;
use feature 'switch';

our @EXPORT_OK = qw(
    GetPriceListScalePrices
    SetPriceListScalePrices
    SetPriceListScalePriceData
    GetPriceListScalePriceData
);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Core::API::Error qw(Error);
use DE_EPAGES::Core::API::Log qw(LogDebug);
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::PriceList::API::Table::ScalePrice qw ( GetScalePricesByPriceListID );
use DE_EPAGES::ShopMigration::API::Constants qw(
    PRICELIST_SCALEPRICE_ATTRIBUTES
);
use DE_EPAGES::PriceList::API::PriceList qw (GetExportablePriceLists);
#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'REFERENCED_PRODUCT_NOT_FOUND' => 'Cannot create scale price for product #Name as product was not found in shop #Shop',
        'REFERENCED_PRICELIST_NOT_FOUND' => 'Cannot create scale price as Pricelist #Name was not found in shop #Shop',
    };
}
#ERRORDEFINITION END

#========================================================================================
# function     GetPriceListScalePrices
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahPriceListScalePrices = GetPriceListScalePrices($Shop);
#----------------------------------------------------------------------------------------
# description  Get the scale prices for a shop
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
#----------------------------------------------------------------------------------------
# return       $ahPriceListScalePrices| see GetPriceListScalePriceData()
#               <ul>
#                   <li>PriceListAlias   - pricelist alias |   - sting
#                   <li>RelativePath - relative product path | - string
#                   <li>Quantity - quantity | - decimal
#                   <li>Price - price | - decimal
#               </ul> | reference to array of hashes
#========================================================================================
sub GetPriceListScalePrices {
    my ($Shop) = @_;
    my @Items;

    my $CurrencyID = $Shop->get('CurrencyID');
    my $PriceLists = GetExportablePriceLists($Shop);

    foreach my $PriceList (@$PriceLists) {
        my $ScalePricesForPriceList = GetScalePricesByPriceListID($Shop->id, $PriceList->id);

        foreach my $ScalePriceEntry (@$ScalePricesForPriceList) {
            my $Product = LoadObject($ScalePriceEntry->{'ProductID'});
            #do not export ScalePrice for subproducts with HasSubOwnPrice=0, possible leftovers in DB
            next if (!$Product->get('HasSubOwnPrices') && defined $Product->get('SuperProduct'));
            # [UNITY-7481] workaround: skip Products without list price
            my $ListPrice = $Product->getListPrice($CurrencyID);
            next unless defined $ListPrice;
            my $hValues = $Product->get(['RelativePath', 'IsDownloadProduct', 'IsBundleProduct', 'IsCustomizable']);
            my $SuperProduct = $Product->superProduct;
            # [UNITY-7812] add non-zero values from master to skip products where this boolean is only set on master
            if (defined $SuperProduct) {
                my $hValuesMaster = $SuperProduct->get(['IsDownloadProduct', 'IsBundleProduct', 'IsCustomizable']);
                my @NonZeros = grep { $hValuesMaster->{$_} == 1; } keys %$hValuesMaster;
                @$hValues{@NonZeros} = @$hValuesMaster{@NonZeros};
            }
            my @FailReason = grep { $hValues->{$_} == 1 && ($_ ne 'RelativePath')  } keys %$hValues;
            if (scalar @FailReason) {
                LogDebug(sprintf("Skip Product '%s' because of %s:%s\n", $Product->pathString, $FailReason[0], '1'), 1);
                next;
            }
            $ScalePriceEntry->{'PriceListAlias'} = $PriceList->alias;
            $ScalePriceEntry->{'RelativePath'} = $hValues->{'RelativePath'};

            push @Items, $ScalePriceEntry;
        }
    }

    return \@Items;
}

#========================================================================================
# function     GetPriceListScalePriceData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hPriceListScalePrice = GetPriceListScalePriceData($Shop, $PriceListScalePrices);
#----------------------------------------------------------------------------------------
# description  Constructs a hash with the exportable fields for a single PriceListScalePrice
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
# input        $PriceListScalePrice | PriceListScalePrice | object
#----------------------------------------------------------------------------------------
# return       $hPriceListScalePrice | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>RelativePath                     - string</li>
#                           <li>PriceListAlias                   - string</li>
#                           <li>Quantity                         - integer</li>
#                           <li>Price                            - float</li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub GetPriceListScalePriceData {
    my ($Shop, $PriceListScalePrices) = @_;
    my %Info = ();

    my $Attributes = [ @{PRICELIST_SCALEPRICE_ATTRIBUTES()} ];
    foreach my $Attribute (@$Attributes) {
        $Info{$Attribute} = $PriceListScalePrices->{$Attribute} if defined $PriceListScalePrices->{$Attribute};
    }
    return \%Info;
}

#========================================================================================
# function     SetPriceListScalePrices
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetPriceListScalePrices($Shop, $ahPriceListScalePrices);
#----------------------------------------------------------------------------------------
# description  sets attributes on PriceListScalePrice as specified in the $ahPriceListScalePrices
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $ahPriceListScalePrices  | see SetPriceListScalePriceData() | reference to array of hashes
#========================================================================================
sub SetPriceListScalePrices {
    my ($Shop, $ahPriceListScalePrices) = @_;

    foreach my $hPriceListScalePrice (@$ahPriceListScalePrices) {
        SetPriceListScalePriceData($Shop, $hPriceListScalePrice);
    }
}

#========================================================================================
# function     SetPriceListScalePriceData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetPriceListScalePriceData($Shop, $hPriceListScalePrice);
#----------------------------------------------------------------------------------------
# description  sets attributes on a PriceListScalePrice as specified in the hash $hPriceListScalePrice
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $hPriceListScalePrice    | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>ProductAlias                     - string</li>
#                           <li>PriceListAlias                   - string</li>
#                           <li>Quantity                         - integer</li>
#                           <li>Price                            - float</li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub SetPriceListScalePriceData {
    my ($Shop, $hPriceListScalePrice) = @_;
    my @Attributes = @{PRICELIST_SCALEPRICE_ATTRIBUTES()};

    foreach my $Key (keys %{$hPriceListScalePrice->{Attributes}}) {
        die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @Attributes);
    }

    my $PriceListAlias;
    if ($Shop->get('IsMigrationPrepShop')) {
        #add temporary Prefix "PL_epages6_" to $PriceListAlias
        $PriceListAlias = 'PL_epages6_'.$hPriceListScalePrice->{'PriceListAlias'};
    } else{
        $PriceListAlias = $hPriceListScalePrice->{'PriceListAlias'};
    }

    my $PriceList = $Shop->folder('PriceLists')->existsChild($PriceListAlias) ?
        $Shop->folder('PriceLists')->child($PriceListAlias) :
        Error('REFERENCED_PRICELIST_NOT_FOUND', { Name => $PriceListAlias, Shop => $Shop->alias });

    my $PrepShopPath = $Shop->get('Path');
    my $Product = LoadObjectByPath($PrepShopPath.'/'.$hPriceListScalePrice->{'RelativePath'});
    Error('REFERENCED_PRODUCT_NOT_FOUND', { Name => $hPriceListScalePrice->{'RelativePath'}, Shop => $Shop->alias }) unless defined $Product;

    if ($Product->get('IsMaster') && $Shop->get('IsMigrationPrepShop')) {
        #get Baseshop for SubProduct handling
        my $BaseShop = $Shop->get('MigrationShop');
        my $BaseShopProduct = LoadObjectByPath($BaseShop->get('Path').'/'.$hPriceListScalePrice->{'RelativePath'});

        #get all SubProducts from Master of Baseshop because of HasSubOwnPrices flag
        foreach my $SubProduct (@{$BaseShopProduct->get('SubProducts')}) {

            if (!$SubProduct->get('HasSubOwnPrices')) {
                #Insert ScalePrice for Supproduct into prepshop
                my $SubProductPrepShop = LoadObjectByPath($PrepShopPath.'/'.$SubProduct->get('RelativePath'));
                LoadClassByAlias('ScalePrice')->insertObject({
                    'PriceList' => $PriceList,
                    'Product'   => $SubProductPrepShop,
                    'Quantity'  => $hPriceListScalePrice->{'Quantity'},
                    'Price'     => $hPriceListScalePrice->{'Price'},
                });
            }
        }
    }

    LoadClassByAlias('ScalePrice')->insertObject({
        'PriceList' => $PriceList,
        'Product'   => $Product,
        'Quantity'  => $hPriceListScalePrice->{'Quantity'},
        'Price'     => $hPriceListScalePrice->{'Price'},
    });

    return;
}

1;
